/**
 * 
 * Message.java
 * 
 * Represents a Chat-In Protocol Message
 * @author i_tsyrin
 */
package marf.net.messaging.protocol;

import java.io.Serializable;
import java.util.Vector;


/**
 * Represents a unit of transmission. 
 * 
 * $Id: Message.java,v 1.1 2006/09/17 20:06:10 mokhov Exp $
 * 
 * @author Serguei Mokhov
 * @since A2
 */
public class Message
implements Serializable, Cloneable
{
	/*
	 * Message types so far.
	 */
	public static final int UNKNOWN = 0;
	public static final int TRADE = 1;
	public static final int TRADE_SUCCESS = 2;
	public static final int ERROR = 3;
	
	/**
	 * Human descriptions of the above messages.
	 * Should obey the exact order of enumeration.
	 */
	public static final String[] MESSAGE_NAMES =
	{
		"UNKNOWN",
		"trade",
		"tradeSuccess",
		"ERROR",
	};
	
	/**
	 * Message's content.
	 */
	private Vector oMessageContent;
	
	/**
	 * Constructs and empty message.
	 */
	public Message()
	{
		this.oMessageContent = new Vector();
	}
	
	public Vector getMessage()
	{
		return this.oMessageContent;
	}

	public void setMessage(Vector poContent)
	{
		this.oMessageContent = poContent;
	}
	
	public void addMessageComponent(Object o)
	{
		this.oMessageContent.add(o);
	}
	
	/**
	 * Converts Message object instance to human-readable string.
	 */
	public String toString()
	{
		String strMessageString = "";
		
		if(isEmpty())
		{
			return "{empty}";
		}

		int iMessageType = ((Integer)oMessageContent.elementAt(0)).intValue();
		
		for(int i = 0; i < oMessageContent.size(); i++)
		{
			// Message type, convert to human-readable
			if(i == 0)
			{
				strMessageString += "[" + MESSAGE_NAMES[iMessageType] + "]";
			}
			else
			{
				strMessageString += "[" + oMessageContent.elementAt(i) + "]";
			}

		}

		return strMessageString;
	}

	/**
	 * Replaces message's component by index.
	 * @param piIndex index
	 * @param poComponent replacement object
	 */
	public void setMessageComponent(int piIndex, Object poComponent)
	{
		this.oMessageContent.set(piIndex, poComponent);
	}
	
	/**
	 * @return true if the message body is empty.
	 */
	public boolean isEmpty()
	{
		return (this.oMessageContent.size() == 0);
	}
	
	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone()
	{
		Message oClone = new Message();
		oClone.oMessageContent = (Vector)this.oMessageContent.clone();
		return oClone;
	}
}

// EOF
