package marf.net.messaging.protocol.tcp;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Vector;

import marf.net.CommunicationException;
import marf.net.messaging.protocol.Message;
import marf.net.messaging.protocol.PassMessage;
import marf.util.BaseThread;


/**
 * The "server side" of the trade message passing.
 * 
 * @author Serguei Mokhov
 * @since A2
 */
public class Receiver
extends PassMessage
{
	/**
	 * Indicates should the server thread be alive or dead
	 */
	private boolean bAlive;
	
	/**
	 * Server port. 
	 */
	private int iPort = DEFAULT_LISTEN_PORT;
	
	/**
	 * Back-reference to do the stock operations.
	 */
//	private  IStockBroker oDelegate = null;
	
	/**
	 * @param poDelegate
	 * @param piPort
	 */
/*	public Receiver(IStockBroker poDelegate, int piPort)
	{
		super();
		
		this.bAlive = true;
		this.oDelegate = poDelegate;
		this.iPort = piPort;

		System.out.println("server (" + getTID() +") constructed with port " + this.iPort + ", delegate: " + this.oDelegate);
	}

	/**
	 * @see java.lang.Thread#start()
	 */
	public void start()
	{
		BaseThread oServerThread = new BaseThread(this);
		oServerThread.setDaemon(true);
		oServerThread.start();
	}

	/**
	 * 
	 */
	public void stopThread()
	{
		this.bAlive = false;
	}

	/**
	 * @see ca.concordia.cse.dsb.server.messaging.protocol.PassMessage#sendMessage(ca.concordia.cse.dsb.server.messaging.protocol.Message, java.net.Socket)
	 */
	public void sendMessage(Message poMessage, Socket poSocket)
	throws CommunicationException
	{
		try
		{
			System.out.println("server: last element " + poMessage.getMessage().lastElement().getClass().getName());
			System.out.println("server: sending " + poMessage);
	
			DataOutputStream out =new DataOutputStream( poSocket.getOutputStream());
			ObjectOutputStream oout = new ObjectOutputStream(out);
			oout.writeObject(poMessage);

			System.out.println("server: sent " + poMessage);
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new CommunicationException(e);
		}
	}

	/**
	 * @see ca.concordia.cse.dsb.server.messaging.protocol.PassMessage#receiveMessage(java.net.Socket)
	 */
	public Message receiveMessage(Socket poSocket)
	throws CommunicationException
	{
		try
		{
			System.out.println("server (" + getTID() + "): receiving");
			
			DataInputStream oIN = new DataInputStream( poSocket.getInputStream());

			System.out.println("server (" + getTID() + "): packet received, converting to Message");

			ObjectInputStream oOIN = new ObjectInputStream(oIN);
			Message oRet = (Message)oOIN.readObject();

			System.out.println("server (" + getTID() +"): message received: [" + oRet + "]");

			return oRet;
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new CommunicationException(e);
		}
	}

	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run()
	{
		try
		{
			System.out.println("server: starts.");
		
			ServerSocket oListenSocket = new ServerSocket(this.iPort);
	
			// An infinite loop while we are alive
			while(this.bAlive)
			{
				if(!this.bAlive)
				{
					break;
				}

				Socket oClientSocket = oListenSocket.accept();
				new ResponseServerThread(oClientSocket).start();
			}
		}
		catch(Exception e)
		{
			System.err.println("server: " + e);
			e.printStackTrace(System.err);
		}
		
		System.out.println("Server: stopped.");
	
	}

	/**
	 * Response server thread handles the actual message processing and reply.
	 * @author Serguei Mokhov
	 * @since A2
	 */
	class ResponseServerThread
	extends BaseThread
	{
		private DataInputStream oIN;
		private DataOutputStream oOUT;
		private Socket oClientSocket;

		/**
		 * @param poClientSocket
		 */
		public ResponseServerThread(Socket poClientSocket)
		{
			try
			{
				oClientSocket = poClientSocket;
				oIN = new DataInputStream(oClientSocket.getInputStream());
				oOUT = new DataOutputStream(oClientSocket.getOutputStream());
			}
			catch(IOException e)
			{
				System.err.println("Error: " + e.getMessage());
				e.printStackTrace(System.err);
			}
		}
		
		public void run()
		{
			try
			{
				Message oRequest = receiveMessage(oClientSocket);
				
				int iType = Integer.parseInt(oRequest.getMessage().firstElement().toString());
				
				switch(iType)
				{
					case Message.TRADE:
					{
						// Reply must be a replica of the request
						// for validity except that the first message component
						// has to be changed to indicate the reply type.
						Message oReply = (Message)oRequest.clone();
						Vector oMessageData = oReply.getMessage();
						
/*						Account oAccount2 = null;
						
						try
						{
							String strStockSym1 = oMessageData.elementAt(2).toString();
							int iQuanity1 = ((Integer)oMessageData.elementAt(3)).intValue();

							int iCustNo2 = ((Integer)oMessageData.elementAt(4)).intValue();
							String strStockSym2 = oMessageData.elementAt(5).toString();
							int iQuanity2 = ((Integer)oMessageData.elementAt(6)).intValue();
							
							oAccount2 = oDelegate.getTradeStockValidAccount(iCustNo2, strStockSym2, iQuanity2);
							oAccount2.addBalance(strStockSym1, iQuanity1);
							oAccount2.getBalance(strStockSym2).subtractBalance(iQuanity2);

							oReply.setMessageComponent(0, new Integer(Message.TRADE_SUCCESS));
						}
						catch(Exception e)
						{
							e.printStackTrace(System.err);
							oReply.setMessageComponent(0, new Integer(Message.ERROR));
							oReply.addMessageComponent(e);
						}
						
						System.out.println("Account 2: " + oAccount2);*/
						sendMessage(oReply, oClientSocket);
						
						break;
					}
					
					case Message.ERROR:
					case Message.TRADE_SUCCESS:
					case Message.UNKNOWN:
					default:
					{
						throw new RuntimeException("Message of wrong type: " + oRequest);
					}
				}
			}
			catch(Exception e)
			{
				System.err.println(e.getMessage());
				e.printStackTrace(System.err);
			}
			finally
			{
				try
				{
					oClientSocket.close();
				}
				catch(IOException e)
				{
					/*close failed*/
					e.printStackTrace(System.err);
				}
			}
		}
	}
}

// EOF
