package marf.net.server.corba.FeatureExtraction;

import marf.FeatureExtraction.FeatureExtractionException;
import marf.net.CommunicationException;
import marf.net.server.corba.CORBACommunicationException;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.Preprocessing.IPreprocessingCORBA;
import marf.net.server.corba.util.MARFObjectAdapter;
import marf.net.server.corba.util.ORBUtils;
import marf.net.server.delegates.FeatureExtraction.BasicFeatureExtractionDelegate;
import marf.net.server.delegates.FeatureExtraction.IFeatureExtractionDelegate;
import marf.util.Arrays;
import marf.util.Logger;
import marf.util.MARFException;

import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.PortableServer.POA;


/**
 * Implements CORBA-specific FeatureExtraction Service Wrapper.
 * 
 * @author mokhov
 *
 */
public class FeatureExtractionServant
extends IFeatureExtractionCORBAPOA
implements ICORBAServer
{
	/**
	 * Business and transaction logic delegate.
	 */
	private IFeatureExtractionDelegate oDelegate = null;
	
	/**
	 * Reference to a remote preprocessing service to query for
	 * the preprocessed sample data.
	 */
	private IPreprocessingCORBA oPreprocessingRemote = null;

	/**
	 * The root POA of this servant. 
	 */
	private POA oRootPOA = null;

	
	/**
	 * 
	 */
	public FeatureExtractionServant()
	throws Exception
	{
		super();
		new Logger("featureextraction.corba.log");
		this.oDelegate = new BasicFeatureExtractionDelegate();
	}

	/**
	 * @see marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAOperations#init(int)
	 */
	public boolean init(int piFeatureExtractionMethod)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.init(piFeatureExtractionMethod);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAOperations#extractFeatures()
	 */
	public boolean extractFeatures()
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.extractFeatures(this.oPreprocessingRemote.getSample().adSample);
		}
		catch(FeatureExtractionException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAOperations#extractFeaturesFromArray(double[])
	 */
	public boolean extractFeaturesFromArray(double[] padData)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.extractFeatures(padData);
		}
		catch(FeatureExtractionException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAOperations#getFeaturesArray()
	 */
	public double[] getFeaturesArray()
	{
		return this.oDelegate.getFeaturesArray();
	}

	/**
	 * @see marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAOperations#getPreprocessing()
	 */
	public IPreprocessingCORBA getPreprocessing()
	{
		return this.oPreprocessingRemote;
	}

	/**
	 * @see marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAOperations#setPreprocessing(marf.net.server.corba.Preprocessing.IPreprocessingCORBA)
	 */
	public void setPreprocessing(IPreprocessingCORBA poPreprocessing)
	{
		this.oPreprocessingRemote = poPreprocessing;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#getRootPOA()
	 */
	public POA getRootPOA()
	{
		return this.oRootPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#setRootPOA(org.omg.PortableServer.POA)
	 */
	public void setRootPOA(POA poPOA)
	{
		this.oRootPOA = poPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#register(org.omg.CosNaming.NamingContext)
	 */
	public void register(NamingContext poNamingContext)
	throws Exception
	{
  		org.omg.CORBA.Object oCORBAServantObject = this.oRootPOA.servant_to_reference(this);
  		IFeatureExtractionCORBA oServantInterface = IFeatureExtractionCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("IFeatureExtractionCORBA", "");
        NameComponent[] aoPath = {oNameComponent};
        poNamingContext.rebind(aoPath, oServantInterface);
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		String[] astrORBArgv =
		{
			"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_FEATURE_EXTRACTION_PORT),
			"-ORBInitialHost", ORBUtils.getFeatureExtractionHost()
		};

		ORBUtils.startORBFeatureExtractionService(Arrays.concatenate(astrORBArgv, argv));
	}
}

// EOF
