/**
 *
 */
package marf.net.server.corba.MARF;

import marf.MARF;
import marf.Classification.ClassificationException;
import marf.gui.WaveGrapher;
import marf.net.CommunicationException;
import marf.net.client.corba.ICORBAClient;
import marf.net.server.corba.CORBACommunicationException;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.Classification.IClassificationCORBA;
import marf.net.server.corba.Classification.IClassificationCORBAHelper;
import marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBA;
import marf.net.server.corba.FeatureExtraction.IFeatureExtractionCORBAHelper;
import marf.net.server.corba.Preprocessing.IPreprocessingCORBA;
import marf.net.server.corba.Preprocessing.IPreprocessingCORBAHelper;
import marf.net.server.corba.Storage.Configuration;
import marf.net.server.corba.Storage.ISampleLoaderCORBA;
import marf.net.server.corba.Storage.ISampleLoaderCORBAHelper;
import marf.net.server.corba.Storage.ModuleParams;
import marf.net.server.corba.Storage.Result;
import marf.net.server.corba.Storage.ResultSet;
import marf.net.server.corba.Storage.Sample;
import marf.net.server.corba.Storage.SampleHolder;
import marf.net.server.corba.util.MARFObjectAdapter;
import marf.net.server.corba.util.ORBUtils;
import marf.net.server.delegates.MARF.BasicMARFDelegate;
import marf.net.server.delegates.MARF.IMARFServerDelegate;
import marf.util.Arrays;
import marf.util.Debug;
import marf.util.Logger;
import marf.util.MARFException;
import marf.util.NotImplementedException;

import org.omg.CORBA.Any;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.PortableServer.POA;


/**
 * Main MARF CORBA Server Pipeline implementation.
 * This implementation is both a client and a server.
 * The former is for the other concrete services, and
 * the latter is a FE for the applications.
 *
 * @author Serguei Mokhov
 */
public class MARFServant
extends IMARFServerCORBAPOA
implements ICORBAServer, ICORBAClient
{
	private IMARFServerDelegate oDelegate = null;

	private byte[] atRawAudioData = null;
	
	private ISampleLoaderCORBA oSampleLoaderRemote = null;
	private IPreprocessingCORBA oPreprocessingRemote = null;
	private IFeatureExtractionCORBA oFeatureExtractionRemote = null;
	private IClassificationCORBA oClassificationRemote = null;

	/**
	 * The root POA of this servant. 
	 */
	private POA oRootPOA = null;
	
	/**
	 * @throws Exception
	 */
	public MARFServant()
	throws Exception
	{
		super();
		new Logger("marf.server.corba.log");
		this.oDelegate = new BasicMARFDelegate();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setPreprocessingMethod(int)
	 */
	public void setPreprocessingMethod(int piPreprocessingMethod)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setPreprocessingMethod(piPreprocessingMethod);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getPreprocessingMethod()
	 */
	public int getPreprocessingMethod()
	{
		return this.oDelegate.getPreprocessingMethod();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setFeatureExtractionMethod(int)
	 */
	public void setFeatureExtractionMethod(int piFeatureExtractionMethod)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setFeatureExtractionMethod(piFeatureExtractionMethod);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getFeatureExtractionMethod()
	 */
	public int getFeatureExtractionMethod()
	{
		return this.oDelegate.getFeatureExtractionMethod();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setClassificationMethod(int)
	 */
	public void setClassificationMethod(int piClassificationMethod)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setClassificationMethod(piClassificationMethod);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getClassificationMethod()
	 */
	public int getClassificationMethod()
	{
		return this.oDelegate.getClassificationMethod();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setSampleFormat(int)
	 */
	public void setSampleFormat(int piSampleFormat)
	{
		this.oDelegate.setSampleFormat(piSampleFormat);
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getSampleFormat()
	 */
	public int getSampleFormat()
	{
		return this.oDelegate.getSampleFormat();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setSampleFile(java.lang.String, byte[])
	 */
	public void setSampleFile(String pstrFileName, byte[] patAudioData)
	{
		System.out.println("Reached setSampleFile...");
		this.oDelegate.setSampleFile(pstrFileName);
		//XXX
		this.atRawAudioData = patAudioData;
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getSampleFile()
	 */
	public String getSampleFile()
	{
		return this.oDelegate.getSampleFile();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setSamplesDir(java.lang.String)
	 */
	public void setSamplesDir(String pstrSamplesDir)
	{
		this.oDelegate.setSamplesDir(pstrSamplesDir);
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setModuleParams(marf.net.server.corba.Storage.ModuleParams)
	 */
	public void setModuleParams(ModuleParams poModuleParams)
	{
		this.oDelegate.setModuleParams(MARFObjectAdapter.getMARFModuleParams(poModuleParams));
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getModuleParams()
	 */
	public ModuleParams getModuleParams()
	{
		return MARFObjectAdapter.getCORBAModuleParams(this.oDelegate.getModuleParams());
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setDumpSpectrogram(boolean)
	 */
	public void setDumpSpectrogram(boolean pbDump)
	{
		this.oDelegate.setDumpSpectrogram(pbDump);
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getDumpSpectrogram()
	 */
	public boolean getDumpSpectrogram()
	{
		return this.oDelegate.getDumpSpectrogram();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setDumpWaveGraph(boolean)
	 */
	public void setDumpWaveGraph(boolean pbDump)
	{
		this.oDelegate.setDumpWaveGraph(pbDump);
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getDumpWaveGraph()
	 */
	public boolean getDumpWaveGraph()
	{
		return this.oDelegate.getDumpWaveGraph();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setCurrentSubject(int)
	 */
	public void setCurrentSubject(int piSubjectID)
	{
		this.oDelegate.setCurrentSubject(piSubjectID);
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getCurrentSubject()
	 */
	public int getCurrentSubject()
	{
		return this.oDelegate.getCurrentSubject();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setSampleLoaderPluginClass(java.lang.String)
	 */
	public void setSampleLoaderPluginClass(String pstrClassName)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setSampleLoaderPluginClass(pstrClassName);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/* (non-Javadoc)
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getSampleLoaderPluginClass()
	 */
	public Any getSampleLoaderPluginClass()
	{
		// TODO Auto-generated method stub
//		return null;
		throw new NotImplementedException();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setPreprocessingPluginClass(java.lang.String)
	 */
	public void setPreprocessingPluginClass(String pstrClassName)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setPreprocessingPluginClass(pstrClassName);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/* (non-Javadoc)
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getPreprocessingPluginClass()
	 */
	public Any getPreprocessingPluginClass()
	{
		// TODO Auto-generated method stub
//		return null;
		throw new NotImplementedException();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setFeatureExtractionPluginClass(java.lang.String)
	 */
	public void setFeatureExtractionPluginClass(String pstrClassName)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setFeatureExtractionPluginClass(pstrClassName);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/* (non-Javadoc)
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getFeatureExtractionPluginClass()
	 */
	public Any getFeatureExtractionPluginClass()
	{
		// TODO Auto-generated method stub
//		return null;
		throw new NotImplementedException();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setClassificationPluginClass(java.lang.String)
	 */
	public void setClassificationPluginClass(String pstrClassName)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setClassificationPluginClass(pstrClassName);
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/* (non-Javadoc)
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getClassificationPluginClass()
	 */
	public Any getClassificationPluginClass()
	{
		// TODO Auto-generated method stub
//		return null;
		throw new NotImplementedException();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getVersion()
	 */
	public String getVersion()
	{
		return this.oDelegate.getVersion();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getIntVersion()
	 */
	public int getIntVersion()
	{
		return this.oDelegate.getIntVersion();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getDoubleVersion()
	 */
	public double getDoubleVersion()
	{
		return this.oDelegate.getDoubleVersion();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getConfig()
	 */
	public String getConfig()
	{
		return this.oDelegate.getConfig();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getSample()
	 */
	public Sample getSample()
	{
		return MARFObjectAdapter.getCORBASample(this.oDelegate.getSample());
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getSampleLoader()
	 */
	public ISampleLoaderCORBA getSampleLoader()
	{
		return this.oSampleLoaderRemote;
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getPreprocessing()
	 */
	public IPreprocessingCORBA getPreprocessing()
	{
		return this.oPreprocessingRemote;
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getFeatureExtraction()
	 */
	public IFeatureExtractionCORBA getFeatureExtraction()
	{
		return this.oFeatureExtractionRemote;
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getClassification()
	 */
	public IClassificationCORBA getClassification()
	{
		return this.oClassificationRemote;
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#queryResultID()
	 */
	public int queryResultID()
	{
		return this.oDelegate.queryResultID();
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getResult()
	 */
	public Result getResult()
	{
		return MARFObjectAdapter.getCORBAResult(this.oDelegate.getResult());
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getResultSet()
	 */
	public ResultSet getResultSet()
	{
		return MARFObjectAdapter.getCORBAResultSet(this.oDelegate.getResultSet());
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#recognize()
	 */
	public void recognize()
	throws CORBACommunicationException
	{
		try
		{
			// LOCAL
			//this.oDelegate.recognize();

			//THIS
			startRecognitionPipeline();

			Debug.debug("MARF: Classifying...");

			synchronized(oClassificationRemote)
			{
				if(oClassificationRemote.classify() == false)
				{
					throw new ClassificationException("Classification returned false.");
				}
			}
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#train()
	 */
	public void train()
	throws CORBACommunicationException
	{
		try
		{
			//LOCAL
			//this.oDelegate.train();

			//THIS
			if(getCurrentSubject() == MARF.UNSET)
			{
				throw new MARFException("Unset subject ID for training.");
			}

			startRecognitionPipeline();

			Debug.debug("MARF: Training...");

			synchronized(oClassificationRemote)
			{
				//XXX batch: soClassification.restore();

				if(oClassificationRemote.train() == false)
				{
					throw new ClassificationException("Training returned false.");
				}

				//XXX batch: soClassification.dump();
			}
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#startRecognitionPipeline()
	 */
	public void startRecognitionPipeline()
	throws CORBACommunicationException
	{
		try
		{
			// LOCAL
			//this.oDelegate.startRecognitionPipeline();

			/*
			 * Checking minimal required settings
			 */
			checkSettings();

			// XXX
			String sstrFileName = getSampleFile();
			boolean sbDumpWaveGraph = MARF.getDumpWaveGraph();

            // Create and initialize the ORB at port number 1500
/*
			Properties oProps = new Properties();
        	oProps.put("org.omg.CORBA.ORBInitialPort", "1500");
        	ORB oORB = ORB.init(argv, oProps);

			org.omg.CORBA.Object oNameService = oORB.resolve_initial_references("NameService");
			NamingContext oNameContext = NamingContextHelper.narrow(oNameService);
			*/

			/*
			 * Sample Loading Stage
			 */
			synchronized(sstrFileName)
			{
				Debug.debug("Loading sample \"" + getSampleFile() + "\"");

				//soSampleLoader = SampleLoaderFactory.create(siSampleFormat);

				// Resolve the Object Reference in Naming
/*
				NameComponent oNaming1 = new NameComponent("ISampleLoaderCORBA", "WAV");
				NameComponent aoPath1 [] = {oNaming1};
				oSampleLoaderRemote = ISampleLoaderCORBAHelper.narrow(oNameContext.resolve(aoPath1));
*/
				String[] astrOBRArgv =
				{
					"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_SAMPLE_LOADER_PORT),
					"-ORBInitialHost", ORBUtils.getSampleLoaderHost()
				};

				oSampleLoaderRemote = ISampleLoaderCORBAHelper.narrow(ORBUtils.startORBSampleLoaderClient(astrOBRArgv));

				synchronized(oSampleLoaderRemote)
				{
//					soSample = soSampleLoader.loadSample(sstrFileName);
//					Sample soSample = oSampleLoaderRemote.loadSample(sstrFileName);
					Sample soSample = oSampleLoaderRemote.loadSampleFromArray(this.atRawAudioData);

					/*
					 * Preprocessing Stage
					 */
					synchronized(soSample)
					{
						Debug.debug("Preprocessing...");
//						soPreprocessing = PreprocessingFactory.create(siPreprocessingMethod, soSample);

						/*
						NameComponent oNaming2 = new NameComponent("IPreprocessingCORBA", "RAW");
						NameComponent aoPath2 [] = {oNaming2};
						oPreprocessingRemote = IPreprocessingCORBAHelper.narrow(oNameContext.resolve(aoPath2));
						*/
						astrOBRArgv = new String[]
						{
							"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_PREPROCESSING_PORT),
							"-ORBInitialHost", ORBUtils.getPreprocessingHost()
						};

						oPreprocessingRemote = IPreprocessingCORBAHelper.narrow(ORBUtils.startORBPreprocessingClient(astrOBRArgv));
						oPreprocessingRemote.init(MARF.getPreprocessingMethod(), new SampleHolder(soSample));

						synchronized(oPreprocessingRemote)
						{
							// TODO: [SM]: Should this be in the preprocessing itself somewhere?

							if(sbDumpWaveGraph)
							{
								Debug.debug("Dumping initial wave graph...");

								new WaveGrapher
								(
//									soSample.getSampleArray(),
									soSample.adSample,
									0,
//									soSample.getSampleArray().length,
									soSample.adSample.length,
									getSampleFile(), //XXX
									"initial"
								).dump();
							}

							Debug.debug("Invoking preprocess() of " + oPreprocessingRemote.getClass().getName());
							oPreprocessingRemote.preprocess();
							Debug.debug("Done preprocess() of " + oPreprocessingRemote.getClass().getName());

							if(sbDumpWaveGraph)
							{
								Debug.debug("Dumping preprocessed wave graph...");

								new WaveGrapher
								(
									soSample.adSample,
									0,
									soSample.adSample.length,
									getSampleFile(),//XXX
									"preprocessed"
								).dump();
							}

							/*
							 * Feature Extraction Stage
							 */
							Debug.debug("Feature extraction...");
//							soFeatureExtraction = FeatureExtractionFactory.create(siFeatureExtractionMethod, soPreprocessing);
/*							NameComponent oNaming3 = new NameComponent("IFeatureExtractionCORBA", "FFT");
							NameComponent aoPath3 [] = {oNaming3};
							oFeatureExtractionRemote = IFeatureExtractionCORBAHelper.narrow(oNameContext.resolve(aoPath3));
*/
							astrOBRArgv = new String[]
							{
								"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_FEATURE_EXTRACTION_PORT),
								"-ORBInitialHost", ORBUtils.getFeatureExtractionHost()
							};

							oFeatureExtractionRemote = IFeatureExtractionCORBAHelper.narrow(ORBUtils.startORBFeatureExtractionClient(astrOBRArgv));
							oFeatureExtractionRemote.setPreprocessing(oPreprocessingRemote);
							oFeatureExtractionRemote.init(MARF.getFeatureExtractionMethod());

							
							//oFeatureExtractionRemote = IFeatureExtractionCORBAHelper.narrow(ORBUtils.resolveObjectReference("IFeatureExtractionCORBA", "FFT"));

							synchronized(oFeatureExtractionRemote)
							{
								oFeatureExtractionRemote.extractFeatures();
//								oFeatureExtractionRemote.extractFeaturesFromArray(oPreprocessingRemote.getSample().adSample);

								/*
								 * Classification Stage
								 */
								Debug.debug("Classification...");
//								soClassification = ClassificationFactory.create(siClassificationMethod, soFeatureExtraction);
/*								NameComponent oNaming4 = new NameComponent("IClassificationCORBA", "CHEBYSHEV_DISTANCE");
								NameComponent aoPath4 [] = {oNaming4};
								oClassificationRemote = IClassificationCORBAHelper.narrow(oNameContext.resolve(aoPath4));*/

								//oClassificationRemote = IClassificationCORBAHelper.narrow(ORBUtils.resolveObjectReference("IClassificationCORBA", "CHEBYSHEV_DISTANCE"));
								astrOBRArgv = new String[]
								{
									"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_CLASSIFICATION_PORT),
									"-ORBInitialHost", ORBUtils.getClassificationHost()
								};
								
								oClassificationRemote = IClassificationCORBAHelper.narrow(ORBUtils.startORBClassificationClient(astrOBRArgv));
								oClassificationRemote.setFeatureExtraction(oFeatureExtractionRemote);
								oClassificationRemote.init(MARFObjectAdapter.getCORBAConfiguration(MARF.getConfiguration()));

								/*
								 * Classification ends in here, as it is continue in one
								 * way or the other in train() or recognize() depending on
								 * the curren run-time mode.
								 */
							}// feat
						}// prep
					}//sample itself
				}//sampleloader
			}//samplefilename

		}
		catch(MARFException e)
		{
			e.printStackTrace(System.err);
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new CORBACommunicationException(e.getMessage());
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#checkSettings()
	 */
	public void checkSettings()
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.checkSettings();
		}
		catch(MARFException e)
		{
			e.printStackTrace(System.err);
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#streamedRecognition()
	 */
	public void streamedRecognition()
	{
		this.oDelegate.streamedRecognition();
	}

	
	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#getConfiguration()
	 */
	public Configuration getConfiguration()
	{
		return MARFObjectAdapter.getCORBAConfiguration(this.oDelegate.getConfiguration());
	}

	/**
	 * @see marf.net.server.corba.MARF.IMARFServerCORBAOperations#setConfiguration(marf.net.server.corba.Storage.Configuration)
	 */
	public Configuration setConfiguration(Configuration poConfiguration)
	throws CORBACommunicationException
	{
		try
		{
			return MARFObjectAdapter.getCORBAConfiguration(this.oDelegate.setConfiguration(MARFObjectAdapter.getMARFConfiguration(poConfiguration)));
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#getRootPOA()
	 */
	public POA getRootPOA()
	{
		return this.oRootPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#setRootPOA(org.omg.PortableServer.POA)
	 */
	public void setRootPOA(POA poPOA)
	{
		this.oRootPOA = poPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#register(org.omg.CosNaming.NamingContext)
	 */
	public void register(NamingContext poNamingContext)
	throws Exception
	{
  		org.omg.CORBA.Object oCORBAServantObject = this.oRootPOA.servant_to_reference(this);
  		IMARFServerCORBA oServantInterface = IMARFServerCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("IMARFServerCORBA", "");
        NameComponent[] aoPath = {oNameComponent};
        poNamingContext.rebind(aoPath, oServantInterface);
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		String[] astrORBArgv =
		{
			"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_MARF_PORT),
			"-ORBInitialHost", ORBUtils.getMARFServerHost()
		};

		ORBUtils.startORBMARFService(Arrays.concatenate(astrORBArgv, argv));
	}
}

// EOF
