/**
 * 
 */
package marf.net.server.corba.Preprocessing;

import marf.Preprocessing.PreprocessingException;
import marf.net.CommunicationException;
import marf.net.server.corba.CORBACommunicationException;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.Storage.Sample;
import marf.net.server.corba.Storage.SampleHolder;
import marf.net.server.corba.util.MARFObjectAdapter;
import marf.net.server.corba.util.ORBUtils;
import marf.net.server.delegates.Preprocessing.BasicPreprocessingDelegate;
import marf.net.server.delegates.Preprocessing.IPreprocessingDelegate;
import marf.util.Arrays;
import marf.util.InvalidSampleFormatException;
import marf.util.Logger;
import marf.util.MARFException;
import marf.util.NotImplementedException;

import org.omg.CORBA.Any;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.PortableServer.POA;


/**
 * CORBA-specific Preprocessing service wrapper.
 * @author mokhov
 *
 */
public class PreprocessingServant
extends IPreprocessingCORBAPOA
implements ICORBAServer
{
	/**
	 * Business and transaction logic delegate.
	 */
	private IPreprocessingDelegate oDelegate = null;
	
	/**
	 * The root POA of this servant. 
	 */
	private POA oRootPOA = null;

	
	/**
	 * 
	 */
	public PreprocessingServant()
	throws Exception
	{
		super();
		new Logger("preprocessing.corba.log");
		this.oDelegate = new BasicPreprocessingDelegate();
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#init(int, marf.net.server.corba.Storage.SampleHolder)
	 */
	public boolean init(int piPreprocessingMethod, SampleHolder poSample)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.init(piPreprocessingMethod, MARFObjectAdapter.getMARFSample(poSample.value));
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#preprocess()
	 */
	public boolean preprocess()
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.preprocess();
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#removeNoise()
	 */
	public boolean removeNoise()
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.removeNoise();
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#removeSilence()
	 */
	public boolean removeSilence()
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.removeSilence();
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#normalize()
	 */
	public boolean normalize()
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.normalize();
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#normalizeFrom(int)
	 */
	public boolean normalizeFrom(int piIndexFrom)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.normalize(piIndexFrom);
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#normalizeRange(int, int)
	 */
	public boolean normalizeRange(int piIndexFrom, int piIndexTo)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.normalize(piIndexFrom, piIndexTo);
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#cropAudio(double, double)
	 */
	public boolean cropAudio(double pdStartingFrequency, double pdEndFrequency)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.cropAudio(pdStartingFrequency, pdEndFrequency);
		}
		catch(PreprocessingException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#getSample()
	 */
	public Sample getSample()
	{
		return MARFObjectAdapter.getCORBASample(this.oDelegate.getSample());
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#setSample(marf.net.server.corba.Storage.Sample)
	 */
	public void setSample(Sample poSample)
	{
		try
		{
			this.oDelegate.setSample(MARFObjectAdapter.getMARFSample(poSample));
		}
		catch(InvalidSampleFormatException e)
		{
			//throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
			e.printStackTrace(System.err);
			throw new RuntimeException(e);
		}
	}

	/**
	 * @see marf.net.server.corba.Preprocessing.IPreprocessingCORBAOperations#_clone()
	 */
	public Any _clone() throws CORBACommunicationException
	{
		// TODO Auto-generated method stub
		//return null;
		throw new NotImplementedException();
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#getRootPOA()
	 */
	public POA getRootPOA()
	{
		return this.oRootPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#setRootPOA(org.omg.PortableServer.POA)
	 */
	public void setRootPOA(POA poPOA)
	{
		this.oRootPOA = poPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#register(org.omg.CosNaming.NamingContext)
	 */
	public void register(NamingContext poNamingContext)
	throws Exception
	{
  		org.omg.CORBA.Object oCORBAServantObject = this.oRootPOA.servant_to_reference(this);
  		IPreprocessingCORBA oServantInterface = IPreprocessingCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("IPreprocessingCORBA", "");
        NameComponent[] aoPath = {oNameComponent};
        poNamingContext.rebind(aoPath, oServantInterface);
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		String[] astrORBArgv =
		{
			"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_PREPROCESSING_PORT),
			"-ORBInitialHost", ORBUtils.getPreprocessingHost()
		};

		ORBUtils.startORBPreprocessingService(Arrays.concatenate(astrORBArgv, argv));
	}
}

// EOF
