/**
 * 
 */
package marf.net.server.corba.Storage;

import marf.Storage.StorageException;
import marf.net.CommunicationException;
import marf.net.server.corba.CORBACommunicationException;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.util.MARFObjectAdapter;
import marf.net.server.corba.util.ORBUtils;
import marf.net.server.delegates.SampleLoader.BasicSampleLoaderDelegate;
import marf.net.server.delegates.SampleLoader.ISampleLoaderDelegate;
import marf.util.Arrays;
import marf.util.InvalidSampleFormatException;
import marf.util.Logger;

import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.PortableServer.POA;


/**
 * @author mokhov
 *
 */
public class SampleLoaderServant
extends ISampleLoaderCORBAPOA
implements ICORBAServer
{
	private ISampleLoaderDelegate oDelegate = null;

	/**
	 * The root POA of this servant. 
	 */
	private POA oRootPOA = null;


	/**
	 * @throws InvalidSampleFormatException
	 * @throws Exception
	 */
	public SampleLoaderServant()
	throws InvalidSampleFormatException, Exception
	{
		super();
		new Logger("sampleloader.corba.log");
		this.oDelegate = new BasicSampleLoaderDelegate();
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#readAudioData(marf.net.server.corba.Storage.AudioDataHolder)
	 */
	public int readAudioData(AudioDataHolder padAudioData)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.readAudioData(padAudioData.value);
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#writeAudioData(double[], int)
	 */
	public int writeAudioData(double[] padAudioData, int piWords)
	throws CORBACommunicationException
	{
		try
		{
			return this.oDelegate.writeAudioData(padAudioData, piWords);
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#loadSample(java.lang.String)
	 */
	public Sample loadSample(String pstrFilename)
	throws CORBACommunicationException
	{
		try
		{
			return MARFObjectAdapter.getCORBASample(this.oDelegate.loadSample(pstrFilename));
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#loadSampleFromArray(byte[])
	 */
	public Sample loadSampleFromArray(byte[] patFileAudioData)
	throws CORBACommunicationException
	{
		try
		{
			return MARFObjectAdapter.getCORBASample(this.oDelegate.loadSample(patFileAudioData));
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#saveSample(java.lang.String)
	 */
	public void saveSample(String pstrFilename)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.saveSample(pstrFilename);
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#updateSample()
	 */
	public void updateSample()
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.updateSample();
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#reset()
	 */
	public void reset()
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.reset();
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#getSampleSize()
	 */
	public int getSampleSize()
	throws CORBACommunicationException
	{
		try
		{
			return (int)this.oDelegate.getSampleSize();
		}
		catch(StorageException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#getSample()
	 */
	public Sample getSample()
	{
		return MARFObjectAdapter.getCORBASample(this.oDelegate.getSample());
	}

	/**
	 * @see marf.net.server.corba.Storage.ISampleLoaderCORBAOperations#setSample(marf.net.server.corba.Storage.Sample)
	 */
	public void setSample(Sample poSample)
	{
		try
		{
			this.oDelegate.setSample(MARFObjectAdapter.getMARFSample(poSample));
		}
		catch(InvalidSampleFormatException e)
		{
			e.printStackTrace(System.err);
			throw new RuntimeException(e);
		}
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#getRootPOA()
	 */
	public POA getRootPOA()
	{
		return this.oRootPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#setRootPOA(org.omg.PortableServer.POA)
	 */
	public void setRootPOA(POA poPOA)
	{
		this.oRootPOA = poPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#register(org.omg.CosNaming.NamingContext)
	 */
	public void register(NamingContext poNamingContext)
	throws Exception
	{
  		org.omg.CORBA.Object oCORBAServantObject = this.oRootPOA.servant_to_reference(this);
  		ISampleLoaderCORBA oServantInterface = ISampleLoaderCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("ISampleLoaderCORBA", "");
        NameComponent[] aoPath = {oNameComponent};
        poNamingContext.rebind(aoPath, oServantInterface);
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		String[] astrORBArgv =
		{
			"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_SAMPLE_LOADER_PORT),
			"-ORBInitialHost", ORBUtils.getSampleLoaderHost()
		};

		ORBUtils.startORBSampleLoaderService(Arrays.concatenate(astrORBArgv, argv));
	}
}

// EOF
