package marf.net.server.corba.util;

import java.util.Vector;

import org.omg.CORBA.Any;

import marf.net.CommunicationException;
import marf.net.server.corba.CORBACommunicationException;
import marf.util.Arrays;
import marf.util.InvalidSampleFormatException;
import marf.util.NotImplementedException;


/**
 * Responsible for translating common MARF data structures to CORBA
 * and vice versa.
 *
 * @author mokhov
 *
 */
public class MARFObjectAdapter
{
	/**
	 *
	 */
	private MARFObjectAdapter()
	{
	}

	/**
	 * @param poCORBAException
	 * @return
	 */
	public static final CommunicationException getMARFCommunicationException(CORBACommunicationException poCORBAException)
	{
		return new CommunicationException(poCORBAException);
	}

	/**
	 * @param poGeneralException
	 * @return
	 */
	public static final CORBACommunicationException getCORBACommunicationException(CommunicationException poGeneralException)
	{
		return new CORBACommunicationException(poGeneralException.getMessage());
	}

	public static final marf.Storage.Result getMARFResult(marf.net.server.corba.Storage.Result poCORBAException)
	{
		return new marf.Storage.Result
		(
			poCORBAException.iID,
			poCORBAException.dOutcome,
			poCORBAException.strDescription
		);
	}

	public static final marf.net.server.corba.Storage.Result getCORBAResult(marf.Storage.Result poGeneralException)
	{
		return new marf.net.server.corba.Storage.Result
		(
			poGeneralException.getID(),
			poGeneralException.getDescription(),
			poGeneralException.getOutcome()
		);
	}

	public static final marf.Storage.ResultSet getMARFResultSet(marf.net.server.corba.Storage.ResultSet poCORBAException)
	{
		marf.Storage.Result[] aoResultSet = new marf.Storage.Result[poCORBAException.oResultSet.length];

		for(int i = 0; i < aoResultSet.length; i++)
		{
			aoResultSet[i] = getMARFResult(poCORBAException.oResultSet[i]);
		}

		marf.Storage.Result[] aoResultSetSorted = new marf.Storage.Result[poCORBAException.aoResultSetSorted.length];

		for(int i = 0; i < aoResultSetSorted.length; i++)
		{
			aoResultSetSorted[i] = getMARFResult(poCORBAException.aoResultSetSorted[i]);
		}

		marf.Storage.ResultSet oResultSet = new marf.Storage.ResultSet
		(
			Arrays.arrayToVector(aoResultSet)
		);

		oResultSet.setSortMode(poCORBAException.iSortMode);
		oResultSet.setResultSetSorted(aoResultSetSorted);

		return oResultSet;
	}

	public static final marf.net.server.corba.Storage.ResultSet getCORBAResultSet(marf.Storage.ResultSet poGeneralException)
	{
		marf.net.server.corba.Storage.Result[] aoResultSet = new marf.net.server.corba.Storage.Result[poGeneralException.getResultSetVector().size()];

		for(int i = 0; i < aoResultSet.length; i++)
		{
			aoResultSet[i] = getCORBAResult((marf.Storage.Result)poGeneralException.getResultSetVector().elementAt(i));
		}

		marf.net.server.corba.Storage.Result[] aoResultSetSorted = new marf.net.server.corba.Storage.Result[poGeneralException.getResultSetSorted().length];

		for(int i = 0; i < aoResultSetSorted.length; i++)
		{
			aoResultSetSorted[i] = getCORBAResult(poGeneralException.getResultSetSorted()[i]);
		}

		return new marf.net.server.corba.Storage.ResultSet
		(
			aoResultSet,
			aoResultSetSorted,
			poGeneralException.getSortMode()
		);
	}

	/**
	 * @param poCORBAException
	 * @return
	 */
	public static final marf.Storage.ModuleParams getMARFModuleParams(marf.net.server.corba.Storage.ModuleParams poCORBAException)
	{
		marf.Storage.ModuleParams oParams = new marf.Storage.ModuleParams();

		if(poCORBAException.oPreprocessingParams != null)
		{
			Vector oPreprocessingParams = new Vector();

			for(int i = 0; i < poCORBAException.oPreprocessingParams.length; i++)
			{
				oPreprocessingParams.add(poCORBAException.oPreprocessingParams[i].extract_Value());
			}

			oParams.setPreprocessingParams(oPreprocessingParams);
		}

		if(poCORBAException.oFeatureExtractionParams != null)
		{
			Vector oFeatureExtractionParams = new Vector();

			for(int i = 0; i < poCORBAException.oFeatureExtractionParams.length; i++)
			{
				oFeatureExtractionParams.add(poCORBAException.oFeatureExtractionParams[i].extract_Value());
			}

			oParams.setFeatureExtractionParams(oFeatureExtractionParams);
		}

		if(poCORBAException.oClassificationParams != null)
		{
			Vector oClassificationParams = new Vector();

			for(int i = 0; i < poCORBAException.oClassificationParams.length; i++)
			{
				oClassificationParams.add(poCORBAException.oClassificationParams[i].extract_Value());
			}

			oParams.setClassificationParams(oClassificationParams);
		}

		return oParams;
	}

	public static final marf.net.server.corba.Storage.ModuleParams getCORBAModuleParams(marf.Storage.ModuleParams poCORBAException)
	{
		marf.net.server.corba.Storage.ModuleParams oParams = new marf.net.server.corba.Storage.ModuleParams();

		oParams.oPreprocessingParams = new Any[0];
		oParams.oFeatureExtractionParams = new Any[0];
		oParams.oClassificationParams = new Any[0];

		/*
		if(poCORBAException.getPreprocessingParams() != null)
		{
			Any[] oPreprocessingParams = new Any[poCORBAException.getPreprocessingParams().size()];

			for(int i = 0; i < poCORBAException.oPreprocessingParams.length; i++)
			{
				oPreprocessingParams.add(poCORBAException.oPreprocessingParams[i].extract_Value());
			}

			oParams.setPreprocessingParams(oPreprocessingParams);
		}

		if(poCORBAException.oFeatureExtractionParams != null)
		{
			Vector oFeatureExtractionParams = new Vector();

			for(int i = 0; i < poCORBAException.oFeatureExtractionParams.length; i++)
			{
				oFeatureExtractionParams.add(poCORBAException.oFeatureExtractionParams[i].extract_Value());
			}

			oParams.setFeatureExtractionParams(oFeatureExtractionParams);
		}

		if(poCORBAException.oClassificationParams != null)
		{
			Vector oClassificationParams = new Vector();

			for(int i = 0; i < poCORBAException.oClassificationParams.length; i++)
			{
				oClassificationParams.add(poCORBAException.oClassificationParams[i].extract_Value());
			}

			oParams.setClassificationParams(oClassificationParams);
		}
		*/

		return oParams;

//		throw new NotImplementedException();
	}
/*
	public static final marf.Storage.MARFAudioFileFormat getMARFAudioFileFormat(marf.net.server.corba.Storage.MARFAudioFileFormat poCORBAException)
	throws InvalidSampleFormatException
	{
		marf.Storage.MARFAudioFileFormat oFormat = new marf.Storage.MARFAudioFileFormat();
		oFormat.setAudioFormat(poCORBAException.iFormat);
		return oFormat;
	}

	public static final marf.net.server.corba.Storage.MARFAudioFileFormat getCORBAAudioFileFormat(marf.Storage.MARFAudioFileFormat poGeneralException)
	{
		marf.net.server.corba.Storage.MARFAudioFileFormat oFormat = new marf.net.server.corba.Storage.MARFAudioFileFormat();
		oFormat.iFormat = poGeneralException.getAudioFormat();
		return oFormat;
	}
*/
	public static final marf.Storage.Sample getMARFSample(marf.net.server.corba.Storage.Sample poCORBASample)
	throws InvalidSampleFormatException
	{
		marf.Storage.Sample oSample = new marf.Storage.Sample(poCORBASample.iFormat);
		oSample.setSampleArray(poCORBASample.adSample);
		return oSample;
	}

	public static final marf.net.server.corba.Storage.Sample getCORBASample(marf.Storage.Sample poGeneralException)
	{
		return new marf.net.server.corba.Storage.Sample(poGeneralException.getAudioFileFormatCode(), poGeneralException.getSampleArray());
	}

	public static final marf.Configuration getMARFConfiguration(marf.net.server.corba.Storage.Configuration poCORBAConfig)
	{
		return new marf.Configuration
		(
			poCORBAConfig.iPreprocessingMethod,
			poCORBAConfig.iFeatureExtractionMethod,
			poCORBAConfig.iClassificationMethod,
			poCORBAConfig.iSampleFormat,
			poCORBAConfig.iCurrentSubject,
			poCORBAConfig.strFileName,
			poCORBAConfig.strSamplesDir,
			getMARFModuleParams(poCORBAConfig.oModuleParams),
			poCORBAConfig.bDumpSpectrogram,
			poCORBAConfig.bDumpWaveGraph,
			poCORBAConfig.strSampleLoaderPluginClass,
			poCORBAConfig.strPreprocessingPluginClass,
			poCORBAConfig.strFeatureExtractionPluginClass,
			poCORBAConfig.strClassificationPluginClass,
			poCORBAConfig.bDebug
		);
	}

	public static final marf.net.server.corba.Storage.Configuration getCORBAConfiguration(marf.Configuration poMARFConfig)
	{
		return new marf.net.server.corba.Storage.Configuration
		(
			poMARFConfig.getPreprocessingMethod(),
			poMARFConfig.getFeatureExtractionMethod(),
			poMARFConfig.getClassificationMethod(),
			poMARFConfig.getSampleFormat(),
			poMARFConfig.getCurrentSubject(),
			poMARFConfig.getFileName(),
			poMARFConfig.getSamplesDir(),
			getCORBAModuleParams(poMARFConfig.getModuleParams()),
			//XXX: null,
			poMARFConfig.isDumpSpectrogramOn(),
			poMARFConfig.isDumpWaveGraphOn(),
			poMARFConfig.getSampleLoaderPluginClass(),
			poMARFConfig.getPreprocessingPluginClass(),
			poMARFConfig.getFeatureExtractionPluginClass(),
			poMARFConfig.getClassificationPluginClass(),
			poMARFConfig.isDebugOn()
		);
	}
}

// EOF
