package marf.net.server.corba.util;

import java.io.FileInputStream;
import java.util.Properties;

import marf.MARF;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.Classification.ClassificationServant;
import marf.net.server.corba.FeatureExtraction.FeatureExtractionServant;
import marf.net.server.corba.MARF.MARFServant;
import marf.net.server.corba.Preprocessing.PreprocessingServant;
import marf.net.server.corba.Storage.SampleLoaderServant;
import marf.net.server.frontend.corba.SpeakerIdent.SpeakerIdentServant;

import org.omg.CORBA.ORB;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.CosNaming.NamingContextHelper;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.NotFound;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;


/**
 * ORB utils for ORB initialization and starting CORBA services.
 *
 * @author Serguei Mokhov
 */
public class ORBUtils
{

	public static final int BASE_ORB_PORT = 10000;

	public static final int BASE_ORB_SPEAKER_IDENT_PORT = BASE_ORB_PORT + 10;
	public static final int BASE_ORB_LANG_IDENT_PORT = BASE_ORB_PORT + 20;
	public static final int BASE_ORB_MARF_PORT = BASE_ORB_PORT;
	public static final int BASE_ORB_SAMPLE_LOADER_PORT = BASE_ORB_PORT + MARF.WAV;
	public static final int BASE_ORB_PREPROCESSING_PORT = BASE_ORB_PORT + MARF.MIN_PREPROCESSING_METHOD;
	public static final int BASE_ORB_FEATURE_EXTRACTION_PORT = BASE_ORB_PORT + MARF.MIN_FEATUREEXTRACTION_METHOD;
	public static final int BASE_ORB_CLASSIFICATION_PORT = BASE_ORB_PORT + MARF.MIN_CLASSIFICATION_METHOD;
	
	/*
speakerident.corba.fe.host=gunnymeden.encs.concordia.ca
marf.corba.server.host=localhost
sampleloader.corba.host=localhost
preprocessing.corba.host=localhost
featureextraction.corba.host=localhost
classification.corba.host=localhost
	*/

	public static final String SPEAKER_IDENT_FE_HOST_PROPERTY = "speakerident.corba.fe.host"; 
	public static final String MARF_SERVER_HOST_PROPERTY = "marf.corba.server.host"; 
	public static final String SAMPLE_LOADER_HOST_PROPERTY = "sampleloader.corba.host"; 
	public static final String PREPROCESSING_HOST_PROPERTY = "preprocessing.corba.host"; 
	public static final String FEATURE_EXTRACTION_HOST_PROPERTY = "featureextraction.corba.host"; 
	public static final String CLASSIFICATION_HOST_PROPERTY = "classification.corba.host"; 
	
	public static final String DEFAULT_HOST = "localhost";
	
	public static final String DEFAULT_HOST_PROPERTIES_FILE = "dmarf-hosts.properties";
	
	public static final Properties SERVICES_HOSTS = new Properties(); 
	
	static
	{
		try
		{
			SERVICES_HOSTS.load(new FileInputStream(DEFAULT_HOST_PROPERTIES_FILE));
			SERVICES_HOSTS.list(System.out);
		}
		catch(Exception e)
		{
			System.err.println("WARNING: " + e.getMessage() + ", using defaults...");

			SERVICES_HOSTS.setProperty(SPEAKER_IDENT_FE_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(MARF_SERVER_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(SAMPLE_LOADER_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(PREPROCESSING_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(FEATURE_EXTRACTION_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(CLASSIFICATION_HOST_PROPERTY, DEFAULT_HOST);
		}
	}
	
	private static NamingContext soNamingContext = null;
	private static ORB soORB = null;

	
	/**
	 * Intentionally empty and private. 
	 */
	private ORBUtils()
	{
	}

	public static String getSpeakerIdentFEHost()
	{
		return SERVICES_HOSTS.getProperty(SPEAKER_IDENT_FE_HOST_PROPERTY);
	}
	
	public static String getMARFServerHost()
	{
		return SERVICES_HOSTS.getProperty(MARF_SERVER_HOST_PROPERTY);
	}
	
	public static String getSampleLoaderHost()
	{
		return SERVICES_HOSTS.getProperty(SAMPLE_LOADER_HOST_PROPERTY);
	}
	
	public static String getPreprocessingHost()
	{
		return SERVICES_HOSTS.getProperty(PREPROCESSING_HOST_PROPERTY);
	}
	
	public static String getFeatureExtractionHost()
	{
		return SERVICES_HOSTS.getProperty(FEATURE_EXTRACTION_HOST_PROPERTY);
	}
	
	public static String getClassificationHost()
	{
		return SERVICES_HOSTS.getProperty(CLASSIFICATION_HOST_PROPERTY);
	}

	/**
	 * @param argv
	 * @return
	 * @throws Exception
	 */
	public static org.omg.CORBA.Object startORBSpeakerIdentClient(String[] argv)
	throws Exception
	{
		NamingContext oClientNamingContext = initializeORB(argv, BASE_ORB_SPEAKER_IDENT_PORT, false);
		return resolveObjectReference(oClientNamingContext, "ISpeakerIdentCORBA", "");
	}
	
	public static org.omg.CORBA.Object startORBMARFClient(String[] argv)
	throws Exception
	{
		NamingContext oClientNamingContext = initializeORB(argv, BASE_ORB_MARF_PORT, false);
		return resolveObjectReference(oClientNamingContext, "IMARFServerCORBA", "");
	}

	public static org.omg.CORBA.Object startORBSampleLoaderClient(String[] argv)
	throws Exception
	{
		NamingContext oClientNamingContext = initializeORB(argv, BASE_ORB_SAMPLE_LOADER_PORT, false);
		return resolveObjectReference(oClientNamingContext, "ISampleLoaderCORBA", "");
	}
	
	public static org.omg.CORBA.Object startORBPreprocessingClient(String[] argv)
	throws Exception
	{
		NamingContext oClientNamingContext = initializeORB(argv, BASE_ORB_PREPROCESSING_PORT, false);
		return resolveObjectReference(oClientNamingContext, "IPreprocessingCORBA", "");
	}
	
	public static org.omg.CORBA.Object startORBFeatureExtractionClient(String[] argv)
	throws Exception
	{
		NamingContext oClientNamingContext = initializeORB(argv, BASE_ORB_FEATURE_EXTRACTION_PORT, false);
		return resolveObjectReference(oClientNamingContext, "IFeatureExtractionCORBA", "");
	}

	public static org.omg.CORBA.Object startORBClassificationClient(String[] argv)
	throws Exception
	{
		NamingContext oClientNamingContext = initializeORB(argv, BASE_ORB_CLASSIFICATION_PORT, false);
		return resolveObjectReference(oClientNamingContext, "IClassificationCORBA", "");
	}

	
	public static NamingContext startORBSpeakerIdentService(String[] argv)
//	throws InvalidName
	throws Exception
	{
		return startORBService(argv, BASE_ORB_SPEAKER_IDENT_PORT, SpeakerIdentServant.class);
	}

	
	public static NamingContext startORBMARFService(String[] argv)
	throws Exception
	{
		return startORBService(argv, BASE_ORB_MARF_PORT, MARFServant.class);
	}
	
	public static NamingContext startORBSampleLoaderService(String[] argv)
	throws Exception
	{
		return startORBService(argv, BASE_ORB_SAMPLE_LOADER_PORT, SampleLoaderServant.class);
	}

	public static NamingContext startORBPreprocessingService(String[] argv)
	throws Exception
	{
		return startORBService(argv, BASE_ORB_PREPROCESSING_PORT, PreprocessingServant.class);
	}

	public static NamingContext startORBFeatureExtractionService(String[] argv)
	throws Exception
	{
		return startORBService(argv, BASE_ORB_FEATURE_EXTRACTION_PORT, FeatureExtractionServant.class);
	}

	public static NamingContext startORBClassificationService(String[] argv)
	throws Exception
	{
		return startORBService(argv, BASE_ORB_CLASSIFICATION_PORT, ClassificationServant.class);
	}

	public static NamingContext initializeORB(String[] argv, int piPort)
	throws InvalidName
	{
		return initializeORB(argv, piPort, true);
	}

	public static NamingContext initializeORB(String[] argv, int piPort, boolean pbServerORB)
	throws InvalidName
	{
        // Create and initialize the ORB at port number 
        Properties oProps = new Properties();
    	oProps.put("org.omg.CORBA.ORBInitialPort", Integer.toString(piPort));
    	ORB oORB = ORB.init(argv, oProps);

		org.omg.CORBA.Object oNameService = oORB.resolve_initial_references("NameService");
		NamingContext oNamingContext = NamingContextHelper.narrow(oNameService);

		if(soNamingContext == null && pbServerORB == true)
		{
			soNamingContext = oNamingContext;
			soORB = oORB;
		}
		
		return oNamingContext;
	}

	public static NamingContext startORBService(String[] argv, int piPort, String pstrServantClass)
//	throws InvalidName, ClassNotFoundException
	throws Exception
	{
		return startORBService(argv, piPort, Class.forName(pstrServantClass));
	}

	public static NamingContext startORBService(String[] argv, int piPort, Class poServantClass)
	//throws InvalidName, AdapterInactive, InstantiationException, IllegalAccessException, WrongPolicy, ServantNotActive, CannotProceed
	throws Exception
	{
		initializeORB(argv, piPort);

        // Get reference to rootpoa & activate the POAManager
  		POA oRootPOA = POAHelper.narrow(soORB.resolve_initial_references("RootPOA"));
 		oRootPOA.the_POAManager().activate();
 		 
        // Create servant - note that we pass it a reference to the POA
 		// XXX
 		//StockBroker oStockBrokerServant = new StockBroker(oRootPOA, iBrokerID);
 		ICORBAServer oServant = (ICORBAServer)poServantClass.newInstance();
 		oServant.setRootPOA(oRootPOA);
                
        // Register the servant with the POA and get object reference for the servant
        // narrow the object reference
        // XXX
//  		org.omg.CORBA.Object oCORBAStockBroker = oRootPOA.servant_to_reference(oStockBrokerServant);
/*
 		org.omg.CORBA.Object oCORBAServantObject = oRootPOA.servant_to_reference((Servant)oServant);
  		IStockBrokerCORBA oCorbaStockBrokerInterface = IStockBrokerCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("IStockBrokerCORBA", "" + iBrokerID);
        NameComponent aoPath[] = {oNameComponent};
        oNameContext.rebind(aoPath, oCorbaStockBrokerInterface);
*/
 		oServant.register(soNamingContext);

 		System.out.println(poServantClass.getName() + " Server ready and waiting...");

        // Wait for invocations from clients
        soORB.run();
        System.out.println(poServantClass.getName() + " ORB has been shut down...");
		
		return soNamingContext;
	}
	
	/**
	 * @param pstrID
	 * @param pstrKind
	 * @return
	 * @throws org.omg.CosNaming.NamingContextPackage.InvalidName
	 * @throws org.omg.CosNaming.NamingContextPackage.NotFound
	 * @throws org.omg.CosNaming.NamingContextPackage.CannotProceed
	 */
	public static org.omg.CORBA.Object resolveObjectReference(NamingContext poNamingContext, String pstrID, String pstrKind)
	throws org.omg.CosNaming.NamingContextPackage.InvalidName, NotFound, CannotProceed
	{
		NameComponent oNaming = new NameComponent(pstrID, pstrKind);
		NameComponent[] aoPath = {oNaming};
		return poNamingContext.resolve(aoPath);
	}
}

// EOF
