/**
 * 
 */
package marf.net.server.delegates.FeatureExtraction;

import java.io.Serializable;

import marf.MARF;
import marf.FeatureExtraction.FeatureExtractionException;
import marf.FeatureExtraction.FeatureExtractionFactory;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.Preprocessing.IPreprocessing;
import marf.util.MARFException;


/**
 * Basic feature extraction delegate for inter-server TCP/UDP communication
 * and transaction processing.
 * 
 * @author Serguei Mokhov
 */
public class BasicFeatureExtractionDelegate
implements IFeatureExtractionDelegate
{
	/**
	 * Core business logic delegate
	 */
	protected IFeatureExtraction oBusinessLogicDelegate = null;

	/**
	 * 
	 */
	public BasicFeatureExtractionDelegate()
	{
		super();
	}

	/**
	 * @see marf.net.server.delegates.FeatureExtraction.IFeatureExtractionDelegate#init(int)
	 */
	public boolean init(int piFeatureExtractionMethod)
	throws MARFException
	{
		MARF.setFeatureExtractionMethod(piFeatureExtractionMethod);
		this.oBusinessLogicDelegate = FeatureExtractionFactory.create(piFeatureExtractionMethod, null);
		return true;
	}

	/**
	 * @see marf.FeatureExtraction.IFeatureExtraction#extractFeatures()
	 */
	public boolean extractFeatures()
	throws FeatureExtractionException
	{
		return this.oBusinessLogicDelegate.extractFeatures();
	}

	/**
	 * @see marf.FeatureExtraction.IFeatureExtraction#extractFeatures(double[])
	 */
	public boolean extractFeatures(double[] padSampleData)
	throws FeatureExtractionException
	{
		return this.oBusinessLogicDelegate.extractFeatures(padSampleData);
	}

	/**
	 * @see marf.FeatureExtraction.IFeatureExtraction#getFeaturesArray()
	 */
	public double[] getFeaturesArray()
	{
		return this.oBusinessLogicDelegate.getFeaturesArray();
	}

	/**
	 * @see marf.FeatureExtraction.IFeatureExtraction#getPreprocessing()
	 */
	public IPreprocessing getPreprocessing()
	{
		return this.oBusinessLogicDelegate.getPreprocessing();
	}

	/**
	 * @see marf.FeatureExtraction.IFeatureExtraction#setPreprocessing(marf.Preprocessing.IPreprocessing)
	 */
	public void setPreprocessing(IPreprocessing poPreprocessing)
	{
		this.oBusinessLogicDelegate.setPreprocessing(poPreprocessing);
	}

	
	
	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#abortTransaction(long)
	 */
	public boolean abortTransaction(long piTxnID) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#beginTransaction()
	 */
	public long beginTransaction(String pstrOperation) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#commitTransaction(long)
	 */
	public boolean commitTransaction(long piTxnID) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#prepareTransaction(long, java.io.Serializable)
	 */
	public boolean prepareTransaction(long piTxnID, Serializable poValue) {
		// TODO Auto-generated method stub
		return false;
	}

}

// EOF