/**
 * 
 */
package marf.net.server.delegates.Preprocessing;

import java.io.Serializable;

import marf.MARF;
import marf.Preprocessing.IPreprocessing;
import marf.Preprocessing.PreprocessingException;
import marf.Preprocessing.PreprocessingFactory;
import marf.Storage.Sample;
import marf.util.MARFException;
import marf.util.NotImplementedException;


/**
 * @author Serguei Mokhov
 */
public class BasicPreprocessingDelegate
implements IPreprocessingDelegate
{
	/**
	 * The business logic is delegated to the actual MARF module.
	 */
	private IPreprocessing oBusinessLogicDelegate = null;

	/**
	 * 
	 */
	public BasicPreprocessingDelegate()
	{
		super();
	}

	/**
	 * @see marf.net.server.delegates.Preprocessing.IPreprocessingDelegate#setPreprocessingMethod(int)
	 */
	public void setPreprocessingMethod(int piPreprocessingMethod)
	throws MARFException
	{
		MARF.setPreprocessingMethod(piPreprocessingMethod);
	}

	/**
	 * @see marf.net.server.delegates.Preprocessing.IPreprocessingDelegate#init(int, marf.Storage.Sample)
	 */
	public boolean init(int piPreprocessingMethod, Sample poSample)
	throws MARFException
	{
		setPreprocessingMethod(piPreprocessingMethod);
		this.oBusinessLogicDelegate = PreprocessingFactory.create(piPreprocessingMethod, poSample);
		System.out.println("Preprocessing initialized.");
		return true;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#abortTransaction(long)
	 */
	public boolean abortTransaction(long piTxnID) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#beginTransaction()
	 */
	public long beginTransaction(String pstrOperation) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#commitTransaction(long)
	 */
	public boolean commitTransaction(long piTxnID) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.delegates.IDelegate#prepareTransaction(long, java.io.Serializable)
	 */
	public boolean prepareTransaction(long piTxnID, Serializable poValue) {
		// TODO Auto-generated method stub
		return false;
	}

	
	
	/**
	 * @see marf.Preprocessing.IPreprocessing#cropAudio(double, double)
	 */
	public boolean cropAudio(double pdStartingFrequency, double pdEndFrequency)
	throws PreprocessingException
	{
		return this.oBusinessLogicDelegate.cropAudio(pdStartingFrequency, pdEndFrequency);
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#getSample()
	 */
	public Sample getSample()
	{
		return this.oBusinessLogicDelegate.getSample();
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#normalize()
	 */
	public boolean normalize()
	throws PreprocessingException
	{
		return this.oBusinessLogicDelegate.normalize();
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#normalize(int, int)
	 */
	public boolean normalize(int piIndexFrom, int piIndexTo)
	throws PreprocessingException
	{
		return this.oBusinessLogicDelegate.normalize(piIndexFrom, piIndexTo);
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#normalize(int)
	 */
	public boolean normalize(int piIndexFrom)
	throws PreprocessingException
	{
		return this.oBusinessLogicDelegate.normalize(piIndexFrom);
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#preprocess()
	 */
	public boolean preprocess()
	throws PreprocessingException
	{
		System.out.println("Preprocessing: " + this.oBusinessLogicDelegate);
		return this.oBusinessLogicDelegate.preprocess();
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#removeNoise()
	 */
	public boolean removeNoise()
	throws PreprocessingException
	{
		return this.oBusinessLogicDelegate.removeNoise();
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#removeSilence()
	 */
	public boolean removeSilence()
	throws PreprocessingException
	{
		return this.oBusinessLogicDelegate.removeSilence();
	}

	/**
	 * @see marf.Preprocessing.IPreprocessing#setSample(marf.Storage.Sample)
	 */
	public void setSample(Sample poSample)
	{
		this.oBusinessLogicDelegate.setSample(poSample);
	}

	public Object clone()
	throws CloneNotSupportedException
	{
		throw new NotImplementedException();
		//XXX return null;
	}
}

// EOF
