/**
 * 
 */
package marf.net.server.frontend.corba.SpeakerIdent;

import marf.MARF;
import marf.Storage.StorageException;
import marf.net.CommunicationException;
import marf.net.client.corba.ICORBAClient;
import marf.net.server.corba.CORBACommunicationException;
import marf.net.server.corba.ICORBAServer;
import marf.net.server.corba.MARF.IMARFServerCORBA;
import marf.net.server.corba.MARF.IMARFServerCORBAHelper;
import marf.net.server.corba.util.MARFObjectAdapter;
import marf.net.server.corba.util.ORBUtils;
import marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate;
import marf.net.server.frontend.delegates.SpeakerIdent.SpeakerIdentDelegate;
import marf.util.Arrays;
import marf.util.Logger;
import marf.util.MARFException;

import org.omg.CosNaming.NameComponent;
import org.omg.CosNaming.NamingContext;
import org.omg.PortableServer.POA;


/**
 * @author mokhov
 *
 */
public class SpeakerIdentServant
extends ISpeakerIdentCORBAPOA
implements ICORBAServer, ICORBAClient
{
	private ISpeakerIdentDelegate oDelegate = new SpeakerIdentDelegate();
	private IMARFServerCORBA oMARFRemote = null;
	
	/**
	 * The root POA of this servant. 
	 */
	private POA oRootPOA = null;


	/**
	 * @throws Exception
	 */
	public SpeakerIdentServant()
	throws Exception
	{
		super();
		new Logger("speakerident.fe.corba.log");
		connectToMARFRemote();
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#start(java.lang.String[])
	 */
	public void start(String[] argv)
	{
		this.oDelegate.start(argv);
	}
	
	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#ident(marf.net.server.corba.Storage.Configuration, java.lang.String, byte[], int)
	 */
	public void ident(marf.net.server.corba.Storage.Configuration poConfig, String pstrFilename, byte[] patSampleData, int piExpectedID)
	throws CORBACommunicationException
	{
		try
		{
			//XXX TXN
			//MARF.setSampleFile(pstrFilename);
			MARF.setConfiguration(MARFObjectAdapter.getMARFConfiguration(poConfig));
			
//			connectToMARFRemote();

			this.oMARFRemote.setSampleFile(pstrFilename, patSampleData);
			this.oMARFRemote.setConfiguration(poConfig);
			
			//MARF.recognize();
			this.oMARFRemote.recognize();

			// First guess
//			int iIdentifiedID = MARF.queryResultID();
			int iIdentifiedID = this.oMARFRemote.queryResultID();

			// Second best
//			int iSecondClosestID = MARF.getResultSet().getSecondClosestID();
			int iSecondClosestID = MARFObjectAdapter.getMARFResultSet(this.oMARFRemote.getResultSet()).getSecondClosestID();

			//int[] aiResultIDs = recognizeAtomic(poConfig, pstrFilename, patSampleData);
			
			
			// Local database and stats management
			this.oDelegate.ident
			(
				MARFObjectAdapter.getMARFConfiguration(poConfig),
				pstrFilename,
				patSampleData, //??? no need in the current model as the delegate does nothing with it?
				piExpectedID,
				iIdentifiedID,
				iSecondClosestID
			);
		}
		catch(Exception e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#train(marf.net.server.corba.Storage.Configuration, java.lang.String, byte[])
	 */
	public void train(marf.net.server.corba.Storage.Configuration poConfig, String pstrFilename, byte[] patSampleData)
	throws CORBACommunicationException
	{
		try
		{
//			synchronized(this.oMARFRemote)
			{
				MARF.setConfiguration(MARFObjectAdapter.getMARFConfiguration(poConfig));
	
//				connectToMARFRemote();
				
				//MARF.setSampleFile(pstrFilename);
				this.oMARFRemote.setSampleFile(pstrFilename, patSampleData);
				this.oMARFRemote.setConfiguration(poConfig);
	
				Integer iID = new Integer(MARF.UNSET);
				
				synchronized(iID)
				{
					iID = new Integer(this.oDelegate.train(MARFObjectAdapter.getMARFConfiguration(poConfig), pstrFilename, patSampleData));
					
					if(iID.equals(new Integer(MARF.UNSET)) == false)
					{
						this.oMARFRemote.setCurrentSubject(iID.intValue());
						this.oMARFRemote.train();
					}
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	private void connectToMARFRemote()
	throws Exception
	{
		if(this.oMARFRemote == null)
		{
			String[] astrOBRArgv =
			{
				"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_MARF_PORT),
				"-ORBInitialHost", ORBUtils.getMARFServerHost()
			};
			
			this.oMARFRemote = IMARFServerCORBAHelper.narrow(ORBUtils.startORBMARFClient(astrOBRArgv));
		}
	}
	
	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#getVersion()
	 */
	public String getVersion()
	{
		return this.oDelegate.getVersion();
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#getIntVersion()
	 */
	public int getIntVersion()
	{
		return this.oDelegate.getIntVersion();
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#validateVersions()
	 */
	public void validateVersions()
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.validateVersions();
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#getConfigString(java.lang.String[])
	 */
	public String getConfigString(String[] pastrArgv)
	{
		return this.oDelegate.getConfigString(pastrArgv);
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#setDefaultConfig()
	 */
	public marf.net.server.corba.Storage.Configuration setDefaultConfig()
	throws CORBACommunicationException
	{
		try
		{
			return MARFObjectAdapter.getCORBAConfiguration(this.oDelegate.setDefaultConfig());
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#setCustomConfig(marf.net.server.corba.Storage.Configuration)
	 */
	public void setCustomConfig(marf.net.server.corba.Storage.Configuration poConfig)
	throws CORBACommunicationException
	{
		try
		{
			this.oDelegate.setCustomConfig(MARFObjectAdapter.getMARFConfiguration(poConfig));
		}
		catch(MARFException e)
		{
			throw MARFObjectAdapter.getCORBACommunicationException(new CommunicationException(e));
		}
	}

	
	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#listBestResultStats()
	 */
	public String[] listBestResultStats()
	{
		return this.oDelegate.listBestResultStats();
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#listStats()
	 */
	public String[] listStats()
	{
		return this.oDelegate.listStats();
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#listTopResultsStats(int)
	 */
	public String[] listTopResultsStats(int piTopN)
	{
		return this.oDelegate.listTopResultsStats(piTopN);
	}

	/**
	 * @see marf.net.server.frontend.corba.SpeakerIdent.ISpeakerIdentCORBAOperations#resetStats()
	 */
	public void resetStats()
	{
		try
		{
			this.oDelegate.resetStats();
		}
		catch(StorageException e)
		{
			e.printStackTrace(System.err);
		}
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#getRootPOA()
	 */
	public POA getRootPOA()
	{
		return this.oRootPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#setRootPOA(org.omg.PortableServer.POA)
	 */
	public void setRootPOA(POA poPOA)
	{
		this.oRootPOA = poPOA;
	}

	/**
	 * @see marf.net.server.corba.ICORBAServer#register(org.omg.CosNaming.NamingContext)
	 */
	public void register(NamingContext poNamingContext)
	throws Exception
	{
  		org.omg.CORBA.Object oCORBAServantObject = this.oRootPOA.servant_to_reference(this);
  		ISpeakerIdentCORBA oServantInterface = ISpeakerIdentCORBAHelper.narrow(oCORBAServantObject);

        // Bind the Object Reference in Naming
        NameComponent oNameComponent = new NameComponent("ISpeakerIdentCORBA", "");
        NameComponent[] aoPath = {oNameComponent};
        poNamingContext.rebind(aoPath, oServantInterface);
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		String[] astrORBArgv =
		{
			"-ORBInitialPort", Integer.toString(ORBUtils.BASE_ORB_SPEAKER_IDENT_PORT),
			"-ORBInitialHost", "localhost"
		};

		ORBUtils.startORBSpeakerIdentService(Arrays.concatenate(astrORBArgv, argv));
	}
}

// EOF
