/**
 * 
 */
package marf.net.server.frontend.delegates.SpeakerIdent;

import java.util.Date;

import marf.Configuration;
import marf.MARF;
import marf.Storage.StorageException;
import marf.net.client.corba.SpeakerIdent.SpeakerIdentApp;
import marf.util.Debug;
import marf.util.MARFException;


/**
 * @author mokhov
 *
 */
public class SpeakerIdentDelegate
implements ISpeakerIdentDelegate
{

	/**
	 * Instance of the database of speakers.
	 */
	protected /*static*/ SpeakersIdentDb soDB = new SpeakersIdentDb("speakers.txt");
	
	/**
	 * 
	 */
	public SpeakerIdentDelegate()
	{
		super();
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#getConfigString(java.lang.String[])
	 */
	public String getConfigString(String[] pastrArgv)
	{
		return SpeakerIdentApp.getConfigString(pastrArgv);
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#getIntVersion()
	 */
	public int getIntVersion()
	{
		return SpeakerIdentApp.getIntVersion();
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#getVersion()
	 */
	public String getVersion()
	{
		return SpeakerIdentApp.getVersion();
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#ident(marf.Configuration, java.lang.String, byte[], int, int, int)
	 */
	public void ident(Configuration poConfig, String pstrFilename, byte[] atSampleData, int piExpectedID, int piIdentifiedID, int piSecondClosestID)
	throws MARFException
	{
		MARF.setConfiguration(poConfig);
		String pstrConfig = MARF.getConfig();
		
		/*
		 * If no expected speaker present on the command line,
		 * attempt to fetch it from the database by filename.
		 */
		if(piExpectedID < 0)
		{
			piExpectedID = soDB.getIDByFilename(pstrFilename, false);
		}

		System.out.println("                 File: " + pstrFilename);
		System.out.println("               Config: " + pstrConfig);
		System.out.println("         Speaker's ID: " + piIdentifiedID);
		System.out.println("   Speaker identified: " + soDB.getName(piIdentifiedID));

		/*
		 * Only collect stats if we have expected speaker
		 */
		if(piExpectedID > 0)
		{
			System.out.println("Expected Speaker's ID: " + piExpectedID);
			System.out.println("     Expected Speaker: " + soDB.getName(piExpectedID));

			soDB.restore();
			{
				// 1st match
				soDB.addStats(pstrConfig, (piIdentifiedID == piExpectedID));

				// 2nd best: must be true if either 1st true or second true (or both :))
				boolean bSecondBest =
					piIdentifiedID == piExpectedID
					||
					piSecondClosestID == piExpectedID;

				soDB.addStats(pstrConfig, bSecondBest, true);
			}
			soDB.dump();
		}

		System.out.println("       Second Best ID: " + piSecondClosestID);
		System.out.println("     Second Best Name: " + soDB.getName(piSecondClosestID));
		System.out.println("            Date/time: " + new Date());
		System.out.println("----------------------------8<------------------------------");
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#setCustomConfig(marf.Configuration)
	 */
	public void setCustomConfig(Configuration poConfig)
	throws MARFException
	{
		MARF.setConfiguration(poConfig);
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#setDefaultConfig()
	 */
	public Configuration setDefaultConfig()
	throws MARFException
	{
		// TODO Auto-generated method stub
		// XXX
		return MARF.getConfiguration();
	}

	/* (non-Javadoc)
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#start(java.lang.String[])
	 */
	public void start(String[] argv) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#train(marf.Configuration, java.lang.String, byte[])
	 */
	public int train(Configuration poConfig, String pstrFilename, byte[] patSampleData)
	throws MARFException
	{
		System.out.println(MARF.getConfiguration());
		System.out.println(poConfig);
		System.out.println(MARF.getConfiguration().toString().equals(poConfig.toString()));

		Debug.enableDebug();

		soDB.connect();
		soDB.query();
		int iID = soDB.getIDByFilename(pstrFilename, true);

		if(iID == -1)
		{
			// XXX gather output for list*() methods
			System.out.println("No speaker found for \"" + pstrFilename + "\" for training.");
		}

		return iID;
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#validateVersions()
	 */
	public void validateVersions()
	throws MARFException
	{
		SpeakerIdentApp.validateVersions();
	}

	/* (non-Javadoc)
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#listBestResultStats()
	 */
	public String[] listBestResultStats() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#listStats()
	 */
	public String[] listStats() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#listTopResultsStats(int)
	 */
	public String[] listTopResultsStats(int piTopN) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * @see marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate#resetStats()
	 */
	public void resetStats()
	throws StorageException
	{
		this.soDB.resetStats();
	}
}

// EOF
