/**
 * 
 */
package marf.net.server.frontend.rmi.SpeakerIdent;

import java.rmi.RemoteException;

import marf.Configuration;
import marf.net.server.rmi.IRMIServer;
import marf.util.MARFException;


/**
 * @author mokhov
 *
 */
public interface ISpeakerIdentRMI
extends IRMIServer
{
	/**
	 * Main body.
	 * @param argv command-line arguments
	 */
	void start(String[] argv)
	throws RemoteException;
	
	/**
	 * Indetifies a speaker using MARF given configuration,
	 * wave filename, and possibly expected speaker ID.
	 *
	 * @param pstrConfig configuration string for stats
	 * @param pstrFilename name of the wave file with voice sample to identify
	 * @param piExpectedID expected speaker ID; if -1 then no stats is kept
	 *
	 * @throws MARFException in case of any error while processing is in MARF
	 */
	void ident(Configuration poConfig, String pstrFilename, byte[] patSampleData, int piExpectedID)
	throws RemoteException, MARFException;

	/**
	 * Updates training set with a new sample from a given file.
	 *
	 * @param pstrFilename name of the wave file with voice sample train the system on
	 *
	 * @throws MARFException in case of any error while processing is in MARF
	 */
	int train(Configuration poConfig, String pstrFilename, byte[] patSampleData)
	throws RemoteException, MARFException;

	/**
	 * Retrieves String representation of the application's version.
	 * @return version String
	 */
	String getVersion()
	throws RemoteException;

	/**
	 * Retrieves integer representation of the application's version.
	 * @return integer version
	 */
	int getIntVersion()
	throws RemoteException;

	/**
	 * Makes sure the applications isn't run against older MARF version.
	 * Exits with 1 if the MARF version is too old.
	 */
	void validateVersions()
	throws RemoteException, MARFException;

	/**
	 * Composes the current configuration of in a string form.
	 *
	 * @param pstrArgv set of configuration options passed through the command line;
	 * can be null or empty. If latter is the case, MARF itself is queried for its
	 * numerical set up inside.
	 *
	 * @return the current configuration setup
	 */
	String getConfigString(String[] pastrArgv)
	throws RemoteException;

	/**
	 * Sets default MARF configuration parameters as normalization
	 * for preprocessing, FFT for feature extraction, Eucludean
	 * distance for training and classification with no spectrogram
	 * dumps and no debug information, assuming WAVE file format.
	 *
	 * @throws MARFException
	 */
	Configuration setDefaultConfig()
	throws RemoteException, MARFException;

	/**
	 * Customizes MARF's configuration based on the options.
	 * @throws MARFException if some options are out of range
	 */
	void setCustomConfig(Configuration poConfig)
	throws RemoteException, MARFException;

	String[] listStats()
	throws RemoteException;

	String[] listBestResultStats()
	throws RemoteException;

	String[] listTopResultsStats(int piTopN)
	throws RemoteException;

	void resetStats()
	throws RemoteException, MARFException;
}

// EOF
