/**
 * 
 */
package marf.net.server.frontend.rmi.SpeakerIdent;

import java.rmi.RemoteException;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.UnicastRemoteObject;

import marf.Configuration;
import marf.MARF;
import marf.net.client.rmi.IRMIClient;
import marf.net.server.frontend.delegates.SpeakerIdent.ISpeakerIdentDelegate;
import marf.net.server.frontend.delegates.SpeakerIdent.SpeakerIdentDelegate;
import marf.net.server.rmi.MARF.IMARFServerRMI;
import marf.net.server.rmi.util.RMIUtils;
import marf.util.MARFException;


/**
 * @author mokhov
 *
 */
public class SpeakerIdentServant
extends UnicastRemoteObject
implements ISpeakerIdentRMI, IRMIClient
{
	/**
	 * Business and transaction logic delegate.
	 */
	private ISpeakerIdentDelegate oDelegate = null;
	
	/**
	 * Remote MARF RMI service for further processing.
	 */
	private IMARFServerRMI oMARFRemote = null;

	
	/**
	 * @throws RemoteException
	 */
	public SpeakerIdentServant()
	throws RemoteException, Exception
	{
		super();
		this.oDelegate = new SpeakerIdentDelegate();
		this.oMARFRemote = RMIUtils.startRMIMARFClient();
	}

	/**
	 * @param port
	 * @throws RemoteException
	 */
	public SpeakerIdentServant(int port) throws RemoteException {
		super(port);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param port
	 * @param csf
	 * @param ssf
	 * @throws RemoteException
	 */
	public SpeakerIdentServant(int port, RMIClientSocketFactory csf,
			RMIServerSocketFactory ssf) throws RemoteException {
		super(port, csf, ssf);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#start(java.lang.String[])
	 */
	public void start(String[] argv)
	throws RemoteException
	{
		this.oDelegate.start(argv);
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#ident(marf.Configuration, java.lang.String, byte[], int)
	 */
	public void ident(Configuration poConfig, String pstrFilename, byte[] patSampleData, int piExpectedID)
	throws MARFException, RemoteException
	{
		//XXX TXN
		//MARF.setSampleFile(pstrFilename);
		MARF.setConfiguration(poConfig);
		
//		connectToMARFRemote();

		this.oMARFRemote.setSampleFile(pstrFilename, patSampleData);
		this.oMARFRemote.setConfiguration(poConfig);
		
		//MARF.recognize();
		this.oMARFRemote.recognize();

		// First guess
//		int iIdentifiedID = MARF.queryResultID();
		int iIdentifiedID = this.oMARFRemote.queryResultID();

		// Second best
//		int iSecondClosestID = MARF.getResultSet().getSecondClosestID();
		int iSecondClosestID = this.oMARFRemote.getResultSet().getSecondClosestID();

		//int[] aiResultIDs = recognizeAtomic(poConfig, pstrFilename, patSampleData);
		
		
		// Local database and stats management
		this.oDelegate.ident
		(
			poConfig,
			pstrFilename,
			patSampleData, //??? no need in the current model as the delegate does nothing with it?
			piExpectedID,
			iIdentifiedID,
			iSecondClosestID
		);
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#train(marf.Configuration, java.lang.String, byte[])
	 */
	public int train(Configuration poConfig, String pstrFilename, byte[] patSampleData)
	throws RemoteException, MARFException
	{
//		synchronized(this.oMARFRemote)
		{
			MARF.setConfiguration(poConfig);

//			connectToMARFRemote();
			
			//MARF.setSampleFile(pstrFilename);
			this.oMARFRemote.setSampleFile(pstrFilename, patSampleData);
			this.oMARFRemote.setConfiguration(poConfig);

			Integer iID = new Integer(MARF.UNSET);
			
			synchronized(iID)
			{
				iID = new Integer(this.oDelegate.train(poConfig, pstrFilename, patSampleData));
				
				if(iID.equals(new Integer(MARF.UNSET)) == false)
				{
					this.oMARFRemote.setCurrentSubject(iID.intValue());
					this.oMARFRemote.train();
				}
			}
			
			return iID.intValue();
		}
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#getVersion()
	 */
	public String getVersion()
	throws RemoteException
	{
		return this.oDelegate.getVersion();
	}

	/* (non-Javadoc)
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#getIntVersion()
	 */
	public int getIntVersion()
	throws RemoteException
	{
		return this.oDelegate.getIntVersion();
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#validateVersions()
	 */
	public void validateVersions()
	throws RemoteException, MARFException
	{
		this.oDelegate.validateVersions();
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#getConfigString(java.lang.String[])
	 */
	public String getConfigString(String[] pastrArgv)
	throws RemoteException
	{
		return this.oDelegate.getConfigString(pastrArgv);
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#setDefaultConfig()
	 */
	public Configuration setDefaultConfig()
	throws RemoteException, MARFException
	{
		return this.oDelegate.setDefaultConfig();
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#setCustomConfig(marf.Configuration)
	 */
	public void setCustomConfig(Configuration poConfig)
	throws RemoteException, MARFException
	{
		this.oDelegate.setCustomConfig(poConfig);
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#listStats()
	 */
	public String[] listStats()
	throws RemoteException
	{
		return this.oDelegate.listStats();
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#listBestResultStats()
	 */
	public String[] listBestResultStats()
	throws RemoteException
	{
		return this.oDelegate.listBestResultStats();
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#listTopResultsStats(int)
	 */
	public String[] listTopResultsStats(int piTopN)
	throws RemoteException
	{
		return this.oDelegate.listTopResultsStats(piTopN);
	}

	/**
	 * @see marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI#resetStats()
	 */
	public void resetStats()
	throws RemoteException, MARFException
	{
		this.oDelegate.resetStats();
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		RMIUtils.startRMISpeakerIdentService();
	}
}

// EOF
