package marf.net.server.gui;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.UIManager;

import marf.MARF;
import marf.gui.util.BorderPanel;
import marf.gui.util.ColoredStatusBox;
import marf.gui.util.ColoredStatusPanel;
import marf.gui.util.GUIThread;
import marf.util.ExpandedThreadGroup;


/**
 * <p>Server Status GUI application to do basic display of
 * server status.</p>
 *
 * $Id: ServerStatus.java,v 1.2 2006/09/17 20:06:25 mokhov Exp $
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.2 $
 */
public class ServerStatus
extends WindowAdapter 
implements ActionListener, ItemListener 
{
	private static final String APPLICATION_TITLE = "MARF Server Component Status";
	private static final String APPLICATION_VERSION = "$Revision: 1.2 $, $Date: 2006/09/17 20:06:25 $"; 
	
	/*
	 * Need the buttons here to be able to distinguish between them
	 * in the listener.
	 */
	 
	private JButton oBrowseButton;
	
	private JTextField oFileLocation;
	
	/**
	 * File Ops
	 */
	private JFileChooser oFileChooser = new JFileChooser();

	/**
	 * Da Meat
	 */
	private static Component oContents;

	private static JFrame oFrame;

	
	
	/**
	 * Da Main.
	 */
	public static void main(String[] argv)
	{
		try 
		{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
//			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		} 
		catch(Exception e) 
		{
			System.err.println(e); 
		}

		// Set all the bijouterie up for out windows and dialogs
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);

		// Create the top-level container and add contents to it.
		oFrame = new JFrame(APPLICATION_TITLE /*+ ": " + getVersion()*/);
		ServerStatus oGUI = new ServerStatus();
		oContents = oGUI.createComponents();

		oFrame.setJMenuBar(oGUI.createMenuBar());
		oFrame.getContentPane().add(oContents, BorderLayout.CENTER);

		// Finish setting up the frame, and show it.
		oFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		oFrame.pack();
		oFrame.setVisible(true);
	}

	/**
	 * Da GUI Meat.
	 * @return The local top-level container Component with all the GUI
	 */
	public Component createComponents() 
	{
		// Title
		BorderPanel oTitlePanel = new BorderPanel(getVersion());
		{
			oTitlePanel.setLayout(new BoxLayout(oTitlePanel, BoxLayout.Y_AXIS));
			
			JPanel oLabelPanel = new JPanel();
			{
				JLabel oTitleLabel = new JLabel
				(
					"<html><font size='6'>" + APPLICATION_TITLE + "</font></html>"
				);
				
				oLabelPanel.add(oTitleLabel);
			}
			
			JPanel oConfigPanel = new JPanel();
			{
				String strConfig = "<html>" + MARF.getConfig().replaceAll("UNSET", "<font color='gray'>UNSET</font>") + "</html>";
				JLabel oUserNameLabel = new JLabel(strConfig);
				oConfigPanel.add(oUserNameLabel);
			}

			oTitlePanel.add(oLabelPanel);
			oTitlePanel.add(oConfigPanel);
		}

		BorderPanel oMajorStatusPanel = new BorderPanel("Overall Liveness Status");
		{
			oMajorStatusPanel.setLayout(new BoxLayout(oMajorStatusPanel, BoxLayout.X_AXIS));
			
			ColoredStatusBox oGreenPanel = new ColoredStatusBox("READY", Color.GREEN);
			ColoredStatusBox oRedPanel = new ColoredStatusBox("DEAD", Color.RED);
			ColoredStatusBox oYellowPanel = new ColoredStatusBox("TXN", Color.YELLOW);
			ColoredStatusBox oOrangePanel = new ColoredStatusBox("2PC", Color.ORANGE);
			ColoredStatusBox oCyanPanel = new ColoredStatusBox("TCP/UDP", Color.CYAN);
			ColoredStatusBox oDarkGrayPanel = new ColoredStatusBox("STOP", Color.DARK_GRAY);

			oMajorStatusPanel.add(oGreenPanel);
			oMajorStatusPanel.add(oRedPanel);
			oMajorStatusPanel.add(oYellowPanel);
			oMajorStatusPanel.add(oOrangePanel);
			oMajorStatusPanel.add(oCyanPanel);
			oMajorStatusPanel.add(oDarkGrayPanel);
		}

		// PDB Location bar
		BorderPanel oLocationPane = new BorderPanel("Log File Location");
		{
			// Need to dupe this for part properties.
			JLabel oHelpLabel = new JLabel("Log File Location: "); 
			oFileLocation = new JTextField(20);
			
			oFileChooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
			
			oBrowseButton = new JButton("Browse...", createImageIcon("images/Open16.gif"));
			oBrowseButton.setMnemonic(KeyEvent.VK_B);
			oBrowseButton.addActionListener(this);
						
			oLocationPane.add(oHelpLabel);
			oLocationPane.add(oFileLocation);
			oLocationPane.add(oBrowseButton);
		}

		// Db Sync Options
		BorderPanel oServerLogPanel = new BorderPanel("Server Log");
		{
			//BorderPanel oConsolePanel = new BorderPanel("Console Panel",new BorderLayout());

			JTextArea oLogText = new JTextArea(18, 40);
			oLogText.setEditable(false);
			JScrollPane oScrollPane = new JScrollPane(oLogText);
			oServerLogPanel.add(oScrollPane, BorderLayout.CENTER);
			
		}
		
		JPanel oMainPane = new JPanel();
		
		oMainPane.setBorder
		(
			BorderFactory.createRaisedBevelBorder()
			//BorderFactory.createBevelBorder(BevelBorder.RAISED)
			//BorderFactory.createBevelBorder(BevelBorder.LOWERED, Color.LIGHT_GRAY, Color.DARK_GRAY)
/*			BorderFactory.createMatteBorder//(//).createEmptyBorder
			(
				10, //top
				10, //left
				10, //bottom
				10, //right
				Color.DARK_GRAY
			)
*/
		);
		
		//pane.setLayout(new GridLayout(0, 1));
		//pane.setLayout(new BorderLayout(0, 1));
		oMainPane.setLayout(new BoxLayout(oMainPane, BoxLayout.Y_AXIS));
		oMainPane.add(oTitlePanel);
		oMainPane.add(oMajorStatusPanel);
		oMainPane.add(oLocationPane);
		oMainPane.add(oServerLogPanel);

		ColoredStatusPanel oColoredStatusPanel = new ColoredStatusPanel();
		oColoredStatusPanel.add(new JLabel("MARF"));
		
		oMainPane.add(oColoredStatusPanel);
		
		return oMainPane;
	}

	/**
	 * Returns the version string with $$ from CVS stripped out.
	 */
	public static String getVersion()
	{
		return APPLICATION_VERSION.replaceAll("\\$", "");
	}

	/**
	 * Collector thread launches all query threads for different chunks
	 * of data and then collects the results for the entire group.
	 */
	private class Collector
	extends GUIThread
	{
		private ExpandedThreadGroup oWorkers;

		public Collector(ExpandedThreadGroup poThreadGroup)
		{
			System.out.println("Collector: has started. ThreadGroup = " + poThreadGroup);
			this.oWorkers = poThreadGroup;
		}
		
		public void run()
		{
			System.out.println("Collector: begins to run.");

			//JDialog oDialog = new JDialog();
			
			try
			{
				//TODO: proper progress dialog.
				//JOptionPane oDialog = new JOptionPane("Blah", JOptionPane.INFORMATION_MESSAGE);
				//JLabel oSomeLable = new JLabel("Progress");
				//oDialog.getContentPane().add(oSomeLable);
				//oDialog.pack();
				//oDialog.setVisible(true);
	
				oWorkers.start();
				oWorkers.join();
			}
			catch(Exception e)
			{
				System.err.println(e);
				e.printStackTrace(System.err);
			}
			finally
			{
				//oDialog.setVisible(false);
			}
			
			//oDialog.showMessageDialog(frame, "sdsdsd");
			System.out.println("Collector: " + this);
			System.out.println("Collector: terminates.");
		}
	}


	private abstract class StatusQueryThread
	extends GUIThread
	{
		public static final int ENCS_ASSETS_USERS_DB  = 0;
		public static final int ENCS_ASSETS_BLD_DB    = 1;
		public static final int ENCS_ASSETS_BRANDS_DB = 2;
		public static final int ENCS_ASSETS_DEPT_DB   = 3;
		public static final int ENCS_ASSETS_PARTS_DB  = 4;
		public static final int ENCS_ASSETS_MODELS_DB = 5;

		protected String strSQL = "";
		//protected ResultSet oResultSet = null;
		protected int iRowCount = 0;
		protected int iDBType = -1;
		protected Vector oDbItems = new Vector();
		protected Object oResults = null;

		public StatusQueryThread(ExpandedThreadGroup poThreadGroup, String pstrSQLString, int piDBType)
		{
			super(poThreadGroup, "SQL:[" + pstrSQLString + ", pdbtype: " + piDBType + "]");
			
			if(piDBType < ENCS_ASSETS_USERS_DB || piDBType > ENCS_ASSETS_PARTS_DB)
				throw new IllegalArgumentException("Database type (piDBType=" + piDBType + ") is out of range.");
			
			if(pstrSQLString == null)
				throw new IllegalArgumentException("The query string is null.");
				
			this.strSQL = pstrSQLString;
			this.iDBType = piDBType;
		}

		public void run()
		{
			System.out.println(this + " has started.");

			try
			{
			}
			catch(Exception e)
			{
				System.err.println(e);
				System.err.println(this);
				e.printStackTrace(System.err);
			}

			System.out.println(this + " has finished. iRowsCount = " + this.iRowCount);
		}

		private Object processResultSet(ResultSet poResultSet) 
		throws SQLException
		{
			this.iRowCount = 0;
				
			poResultSet.beforeFirst();
				
			while(poResultSet.next())
			{
				this.iRowCount++;
				collectDbItems(poResultSet);
			}
			
			return this.oDbItems.toArray();
		}

		public abstract void collectDbItems(ResultSet poResultSet) throws SQLException;

/*		public ResultSet getResultSet()
		{
			return this.oResultSet;
		}
*/		
		public Object getResults()
		{
			return this.oResults;
		}
		
		public int getRowCount()
		{
			return this.iRowCount;
		}
	}

	/**
	 * Knows how to query the users table.
	 */
	public class UsersQueryThread extends StatusQueryThread
	{
		public UsersQueryThread(ExpandedThreadGroup poThreadGroup)
		{
			super
			(
				poThreadGroup,
				"SELECT user, pass FROM inventory_users;",
				ENCS_ASSETS_USERS_DB
			);
		}
		
		public void collectDbItems(ResultSet poResultSet)
		throws SQLException
		{
/*			this.oDbItems.add
			(
				new ENCSAssetsUsersDB
				(
					poResultSet.getString("user"), 
					poResultSet.getString("pass")
				)
			);*/
		}
	}

	/**
	 * Knows how to query the departments.
	 */
	public class DeptQueryThread extends StatusQueryThread
	{
		public DeptQueryThread(ExpandedThreadGroup poThreadGroup)
		{
			super
			(
				poThreadGroup,
				"SELECT dept_code, short_description FROM dw_dept_map ORDER BY short_description;",
				ENCS_ASSETS_DEPT_DB
			);
		}
		
		public void collectDbItems(ResultSet poResultSet)
		throws SQLException
		{
/*			this.oDbItems.add
			(
				new ENCSAssetsDeptDB
				(
					poResultSet.getString("dept_code"), 
					poResultSet.getString("short_description")
				)
			);*/
		}
	}

	/**
	 * Knows how to query the buildings.
	 */
	public class BldQueryThread extends StatusQueryThread
	{
		public BldQueryThread(ExpandedThreadGroup poThreadGroup)
		{
			super
			(
				poThreadGroup,
				"SELECT DISTINCT bld FROM rooms ORDER BY bld;",
				ENCS_ASSETS_BLD_DB
			);
		}
		
		public void collectDbItems(ResultSet poResultSet)
		throws SQLException
		{
			//this.oDbItems.add(new ENCSAssetsBldDB(poResultSet.getString("bld")));
		}
	}

	/**
	 * Knows how to query manufacturers/brands.
	 */
	public class BrandQueryThread extends StatusQueryThread
	{
		public BrandQueryThread(ExpandedThreadGroup poThreadGroup)
		{
			super
			(
				poThreadGroup,
				"SELECT DISTINCT property_value FROM part_properties WHERE property_name = 'Brand' ORDER BY property_value;",
				ENCS_ASSETS_BRANDS_DB
			);
		}
		
		public void collectDbItems(ResultSet poResultSet)
		throws SQLException
		{
//			this.oDbItems.add(new ENCSAssetsBrandsModelsDB(poResultSet.getString("property_value")));
		}
	}

	/**
	 * ActionListener implementation.
	 * @param e ActionEvent
	 */
	public void actionPerformed(ActionEvent e) 
	{
		try
		{
			/*
			 * This happens when we open what we think is a PDB file
			 * from the file chooser UI.
			 */
			if(e.getSource() == oBrowseButton) 
			{
				int iRetVal = oFileChooser.showOpenDialog(ServerStatus.oContents);
	
				if(iRetVal == JFileChooser.APPROVE_OPTION)
				{
					/*
					 * Here we open the ENCSAssetsDB
					 */
					File oFile = oFileChooser.getSelectedFile();
					System.out.println("Opening: " + oFile.getName());
					oFileLocation.setText(oFile.getCanonicalPath());
					
					FileInputStream oFileInputStream = new FileInputStream(oFile);
					
//					this.oInventoryRecordsDB = new PDBDatabase(oFileInputStream);

//					int iRecordCount = this.oInventoryRecordsDB.getRecordCount();
					//System.out.println("Records in the PalmDB: " + iRecordCount + ", creator: " + this.oInventoryRecordsDB.getCreator());

//					oPartsVector = new Vector();

//					for(int i = 0; i < /*iRecordCount*/ 10; i++)
//					for(int i = 0; i < iRecordCount; i++)
					{
//						ENCSAssetsDB oDBItem = new ENCSAssetsDB(this.oInventoryRecordsDB.getRecord(i));
//						oPartsVector.add(oDBItem);
						
						// TODO: fix Kludge
						// Here we assign a sequence number to our items
						// This actis like a displacement to be added to
						// the base object ID or asset tag.
						//{
						//	oDBItem.local_obj_id = i;
						//	oDBItem.local_asset_tag = i;
						//}
						
//						System.out.println("\nrecord " + i + ": " + oDBItem);
/*
					
						ENCSAssetsPartsDB oPartDBItem;
						//Record oRecord = new Record();
						oPartDBItem = new ENCSAssetsPartsDB();
						
						oPartDBItem.created_by = oDBItem.created_by;
						oPartDBItem.part_created = oDBItem.record_created;
						oPartDBItem.part_id = oDBItem.part_id;
						oPartDBItem.local_asset_tag = oDBItem.local_asset_tag;
						
						oPartDBItem.property_name = "Brand";
						oPartDBItem.property_value = "DELL";
						System.out.println("Property for part " + i + ": " + oPartDBItem);
						oPartPropertiesVector.add(oPartDBItem);

						oPartDBItem = new ENCSAssetsPartsDB(oPartDBItem);
						oPartDBItem.property_name = "Model";
						oPartDBItem.property_value = "2001FP";
						System.out.println("Property for part " + i + ": " + oPartDBItem);
						oPartPropertiesVector.add(oPartDBItem);

						oPartDBItem = new ENCSAssetsPartsDB(oPartDBItem);
						oPartDBItem.property_name = "Type";
						oPartDBItem.property_value = "Hybrid flat panel (LCD)";
						System.out.println("Property for part " + i + ": " + oPartDBItem);
						oPartPropertiesVector.add(oPartDBItem);

						oPartDBItem = new ENCSAssetsPartsDB(oPartDBItem);
						oPartDBItem.property_name = "Screen Size";
						oPartDBItem.property_value = "20";
						System.out.println("Property for part " + i + ": " + oPartDBItem);
						oPartPropertiesVector.add(oPartDBItem);
						///this.oPartPropertiesDB.add();
*/
					}


					/*
					 * Here we open the ENCSAssetsPartPropertiesDB presumably located
					 * in the same path as ENCSAssetsDB
					 */
					String strPartsFilename = oFile.getParent() + File.separator + "ENCSAssetsPartPropertiesConU.PDB";
					System.out.println("Opening: " + strPartsFilename);
					FileInputStream oPartsFileInputStream = new FileInputStream(strPartsFilename);

//					this.oPartPropertiesDB = new PDBDatabase(oPartsFileInputStream);
				
					//int iPartsRecordCount = oPartPropertiesDB.getRecordCount();
					//System.out.println("Records in the PartsDB: " + iPartsRecordCount + ", creator: " + oPartPropertiesDB.getCreator());
				
					//oPartPropertiesVector = new Vector();

					//for(int i = 0; i < iPartsRecordCount; i++)
					{
						//ENCSAssetsPartsDB oDBItem = new ENCSAssetsPartsDB(this.oPartPropertiesDB.getRecord(i));
						//oPartPropertiesVector.add(oDBItem);
						//System.out.println("\nproperty " + i + ": " + oDBItem);
					}
				} 
				else 
				{
					System.out.println("Open command cancelled by user.");
				}
			}

			/*
			 * Synch stuff from a .pdb file to CSAssets' app db 
			 * (either 'stuart', or 'cs_assets', or both.
			 */
/*			else if(e.getSource() == this.oGoToCSAssetsButton) 
			{
				try
				{
					boolean bStuart = oStuart.isSelected();
					boolean bCSAssets = oCSAssets.isSelected();
					
					String strSync = "none";
					
					if(bStuart)
					{
						strSync = "stuart";
					}
					
					if(bCSAssets)
					{
						strSync = strSync.equals("none") ? "cs_assets" : strSync + ", cs_assets";
					}
					
					System.out.println("Synch to: " + strSync);

					if(bStuart == false && bCSAssets == false)
					{
						System.err.println("No db has been chosen to import to.");
						return;
					}

					//int iItemCount = this.oInventoryRecordsDB.getRecordCount();
	
					int iBaseAssetTag;
					int iBaseObjectID;
					int iBaseWarrantyID;
					
					Class.forName(strDriver);


					/*
					 * For all the dbs peform the update. 
					 */
					 
	/*				Vector oDbs = new Vector(); 
					
					if(bStuart == true)
					{
						oDbs.add(this.strDevDB);
					}
					
					if(bCSAssets == true)
					{
						oDbs.add(this.strProductionDB);
					}

					for(int k = 0; k < oDbs.size(); k++)
					{
						this.strDBUser = this.oUserNameField.getText();
						this.strDBPassword = new String(this.oPasswordField.getPassword());
						
						Connection oCon = DriverManager.getConnection
						(
							oDbs.elementAt(k).toString(),
							this.strDBUser,
							this.strDBPassword
						);
	
						Statement oStatement = oCon.createStatement
						(
							ResultSet.TYPE_SCROLL_INSENSITIVE,
							ResultSet.CONCUR_UPDATABLE
						);
	
						/*
						 * This chunk locks the Counters table to atomically
						 * reserve needed number of object IDs, asset tags, etc.
						 * to be used in the follow up INSERT queries. The INSERT
						 * queries themselves do not require any table locking. 
						 */
	/*
						String strSQL = "LOCK TABLES Counters WRITE;";
						oStatement.executeUpdate(strSQL);
	
						strSQL = "SELECT * FROM Counters WHERE Counter_Name IN ('assetTag', 'objectID', 'warrantyID') ORDER BY Counter_Name;";
						ResultSet oResultSet = oStatement.executeQuery(strSQL);
		
						oResultSet.beforeFirst();
						{
							oResultSet.next();
							iBaseAssetTag = oResultSet.getInt("ID") + 1;
			
							oResultSet.next();
							iBaseObjectID = oResultSet.getInt("ID") + 1;
		
							oResultSet.next();
							iBaseWarrantyID = oResultSet.getInt("ID") + 1;
						}
						oResultSet.close();
	
//						strSQL =
//							"UPDATE Counters SET ID = ID + " + (iItemCount + 1) +
//							" WHERE Counter_Name IN ('assetTag', 'objectID', 'warrantyID');";

						System.out.println(strSQL);
						oStatement.executeUpdate(strSQL);
	
						strSQL = "UNLOCK TABLES;";
						oStatement.executeUpdate(strSQL);

						
						/// XXXX
//						IImportPlugin oImportPlugin = (IImportPlugin)Class.forName(CURRENT_IMPORT_PLUGIN).newInstance();
//						oImportPlugin.setPartProperties(this.oPartPropertiesVector);
//						oImportPlugin.importData(iBaseAssetTag, iBaseObjectID, iBaseWarrantyID, oStatement, this.oInventoryRecordsDB, this.oPartsVector);
	
						oCon.close();
						
						System.out.println
						(
							"Supposedly, the data is in the *_palm tables for " + oDbs.elementAt(k) + ".\n" +
							"Use now CSAssets interface to resolve the conflicts, if any.\n"
						);
					} // for all dbs
				}
				catch(NullPointerException oException)
				{
					System.err.println("No PDB file was open yet.");
				}
				catch(Exception oException)
				{
					System.err.println(oException);
					System.err.println(this);
					oException.printStackTrace(System.err);
				}
			}
		*/	
			/*
			 * Grab stuff from the cs_assets db,
			 * package it to a .pdb format and save.
			 */
/*			else if(e.getSource() == oGoToPalmButton) 
			{
				System.out.println("Synch to Palm.");

				boolean bUsers = oUsers.isSelected();
				boolean bBld = oBld.isSelected();
				boolean bRoom = oRoom.isSelected();
				boolean bDept = oDept.isSelected();
				boolean bParts = oParts.isSelected();
				boolean bBrandsModels = oBrandsModels.isSelected();
				
				UsersQueryThread oUsersQueryThread;
				BldQueryThread oBldQueryThread;
				DeptQueryThread oDeptQueryThread;
				BrandQueryThread oBrandQueryThread;
				
//				ENCSAssetsUsersDB[] oUsersDB = null;
				
				ExpandedThreadGroup oQueryThreads = new ExpandedThreadGroup("Queries");
				
				Vector oQueryThreadsVector = new Vector();
				
				if(bUsers)
				{
					oUsersQueryThread = new UsersQueryThread(oQueryThreads);
					oQueryThreadsVector.add(oUsersQueryThread);
				}

				if(bBld)
				{
					oBldQueryThread = new BldQueryThread(oQueryThreads);
					oQueryThreadsVector.add(oBldQueryThread);
				}

				/*
				if(bRoom)
				{
					String strSelect = "SELECT location_id, location_long_name FROM rooms ORDER BY location_long_name;";
					new QueryThread(oQueryThreads, strSelect);
				}
*/				
/*
				if(bDept)
				{
					oDeptQueryThread = new DeptQueryThread(oQueryThreads);
					oQueryThreadsVector.add(oDeptQueryThread);
				}

				if(bBrandsModels)
				{
					oBrandQueryThread = new BrandQueryThread(oQueryThreads);
					oQueryThreadsVector.add(oBrandQueryThread);
				}

				Collector oCollector = new Collector(oQueryThreads);
				oCollector.start();
				
				// TODO: Add progress dialog thread start() and join()
				
				try
				{
					oCollector.join();
				}
				catch(InterruptedException ie)
				{
					System.err.println(ie);
					ie.printStackTrace(System.err);
				}
				
				System.out.println("Collector has collected the results.");

				if(bBld)
				{
					// TODO: fix hardcoded 0, should find something more dynamic, maybe use Hashtable
					oBldQueryThread = ((BldQueryThread)oQueryThreadsVector.elementAt(0));
					
					System.out.println("Buildings got: " + oBldQueryThread.getRowCount());
						
					Object[] aoResults = (Object[])oBldQueryThread.getResults();
											
//					PDBDatabase oBldPDB = new PDBDatabase("ENCSAssetsBldConU", "BLDS", "ConU"); 

					for(int i = 0; i < aoResults.length; i++)
					{
//						ENCSAssetsBldDB oBldItem = (ENCSAssetsBldDB)aoResults[i];
						
						//System.out.println(oBldItem);
						
						// This piece of code to create a PDB db.
//						oBldPDB.addRecord(oBldItem.getRecord());
					}
					
					FileOutputStream oFOS = new FileOutputStream("ENCSAssetsBldConU.pdb");
					//oBldPDB.write(oFOS);
					oFOS.close();
				}

				if(bDept)
				{
					// TODO: fix hardcoded 0, should find something more dynamic, maybe use Hashtable
					oDeptQueryThread = ((DeptQueryThread)oQueryThreadsVector.elementAt(0));
					
					System.out.println("Departments got: " + oDeptQueryThread.getRowCount());
						
					Object[] aoResults = (Object[])oDeptQueryThread.getResults();
											
					//PDBDatabase oBldPDB = new PDBDatabase("ENCSAssetsDeptConU", "DEPT", "ConU"); 

					for(int i = 0; i < aoResults.length; i++)
					{
						//ENCSAssetsDeptDB oBldItem = (ENCSAssetsDeptDB)aoResults[i];
						
						//System.out.println(oBldItem);
						
						// This piece of code to create a PDB db.
						//oBldPDB.addRecord(oBldItem.getRecord());
					}
					
					FileOutputStream oFOS = new FileOutputStream("ENCSAssetsDeptConU.pdb");
					//oBldPDB.write(oFOS);
					oFOS.close();
				}

				if(bBrandsModels)
				{
					// TODO: fix hardcoded 0, should find something more dynamic, maybe use Hashtable
					oBrandQueryThread = ((BrandQueryThread)oQueryThreadsVector.elementAt(0));
					
					System.out.println("Brands got: " + oBrandQueryThread.getRowCount());
						
					Object[] oResults = (Object[])oBrandQueryThread.getResults();
											
					//PDBDatabase oBrandPDB = new PDBDatabase("ENCSAssetsBrandsConU", "BRND", "ConU"); 

					for(int i = 0; i < oResults.length; i++)
					{
						//ENCSAssetsBrandsModelsDB oBrandItem = (ENCSAssetsBrandsModelsDB)oResults[i];
						
						//System.out.println(oBrandItem);
						
						// This piece of code to create a PDB db.
						//oBrandPDB.addRecord(oBrandItem.getRecord());
					}
					
					FileOutputStream fos = new FileOutputStream("ENCSAssetsBrandsConU.pdb");
					//oBrandPDB.write(fos);
					fos.close();
				}
			}*/
			else
			{
				System.err.println("WARNING: Unhandled Event: " + e);
			}
		}
		catch(IOException ioe)
		{
			System.err.println(ioe);
			ioe.printStackTrace(System.err);
		}
	}

	public void itemStateChanged(ItemEvent e) 
	{
		System.err.println("Item state changed, but not handled: " + e);
	}

	public JMenuBar createMenuBar() 
	{
		JMenuBar menuBar;
		JMenu menu, submenu;
		JMenuItem menuItem;

		//Create the menu bar.
		menuBar = new JMenuBar();

		//Build the first menu.
		menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_F);
		menu.getAccessibleContext().setAccessibleDescription("Log file operations.");
		menuBar.add(menu);

		//a group of JMenuItems
		menuItem = new JMenuItem("Open...", KeyEvent.VK_O);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, ActionEvent.ALT_MASK));
		menuItem.getAccessibleContext().setAccessibleDescription("Opens log file for writing.");
		menuItem.addActionListener(this);
		menu.add(menuItem);

		//ImageIcon icon = createImageIcon("images/middle.gif");
		//menuItem = new JMenuItem("Both text and icon", icon);
		menuItem = new JMenuItem("Exit");
		menuItem.setMnemonic(KeyEvent.VK_B);
		menuItem.addActionListener(this);
		menu.add(menuItem);
/*
		menuItem = new JMenuItem(icon);
		menuItem.setMnemonic(KeyEvent.VK_D);
		menuItem.addActionListener(this);
		menu.add(menuItem);
*/
		//a group of radio button menu items
/*		menu.addSeparator();
		ButtonGroup group = new ButtonGroup();

		rbMenuItem = new JRadioButtonMenuItem("A radio button menu item");
		rbMenuItem.setSelected(true);
		rbMenuItem.setMnemonic(KeyEvent.VK_R);
		group.add(rbMenuItem);
		rbMenuItem.addActionListener(this);
		menu.add(rbMenuItem);

		rbMenuItem = new JRadioButtonMenuItem("Another one");
		rbMenuItem.setMnemonic(KeyEvent.VK_O);
		group.add(rbMenuItem);
		rbMenuItem.addActionListener(this);
		menu.add(rbMenuItem);

		//a group of check box menu items
		menu.addSeparator();
		cbMenuItem = new JCheckBoxMenuItem("A check box menu item");
		cbMenuItem.setMnemonic(KeyEvent.VK_C);
		cbMenuItem.addItemListener(this);
		menu.add(cbMenuItem);

		cbMenuItem = new JCheckBoxMenuItem("Another one");
		cbMenuItem.setMnemonic(KeyEvent.VK_H);
		cbMenuItem.addItemListener(this);
		menu.add(cbMenuItem);

		//a submenu
		menu.addSeparator();
		submenu = new JMenu("A submenu");
		submenu.setMnemonic(KeyEvent.VK_S);

		menuItem = new JMenuItem("An item in the submenu");
		menuItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_2, ActionEvent.ALT_MASK));
		menuItem.addActionListener(this);
		submenu.add(menuItem);

		menuItem = new JMenuItem("Another item");
		menuItem.addActionListener(this);
		submenu.add(menuItem);
		menu.add(submenu);
*/

		//Build second menu in the menu bar.
		menu = new JMenu("About");
		menu.setMnemonic(KeyEvent.VK_N);
		menu.getAccessibleContext().setAccessibleDescription("About MARF Server GUI");
		menuBar.add(menu);
		return menuBar;
	}

	/**
	 * Returns an ImageIcon, or null if the path was invalid. 
	 */
	protected static ImageIcon createImageIcon(String path) 
	{
		java.net.URL imgURL = ServerStatus.class.getResource(path);

		if(imgURL != null) 
		{
			return new ImageIcon(imgURL);
		} 
		else 
		{
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
}

// EOF
