package marf.net.server.persistence;

import java.io.Serializable;
import java.lang.reflect.Method;
import java.util.Date;


/**
 * Represents a log entry for WAL.
 * Contains the value, status, timestamps, and transaction ID.
 * 
 * @author mokhov
 *
 */
public class Transaction
implements Serializable
{
	public static final int TXN_IDLE = 0;
	public static final int TXN_PENDING = 1;
	public static final int TXN_COMMITTED = 2;
	public static final int TXN_ABORTED = 3;
	
	protected int iStatus = TXN_IDLE;

	//protected Object oValue = null;
	/**
	 * Represents the value of the transactional object.
	 * Must be serializable. Contains the state of the
	 * object as it was to be committed to the main storage.
	 * Can be anything of the form of Serializable (e.g. small
	 * data structures or messages) or of IStorageManager, where
	 * the whole module state is serialized into.
	 */
	protected Serializable oValue = null;
	
	protected String strFilename = "";
	
	/**
	 * What to do when transaction is recovered from the WAL. 
	 */
	protected int iAction;

	//Method oAction = null;
	
	protected Date oBeginTimestamp = new Date();
	protected Date oEndTimestamp = null;
	
	
//	protected int iTransactionID;
	protected long iTransactionID;
	
	public Transaction() {
		super();
		// TODO Auto-generated constructor stub
	}

	public synchronized void execute()
	{
		begin();
		commit();
	}

	public synchronized void begin()
	{
		
	}

	public synchronized void commit()
	{
		
	}

	/**
	 * @return Returns the iAction.
	 */
	public synchronized int getAction() {
		return iAction;
	}

	/**
	 * @param action The iAction to set.
	 */
	public synchronized void setAction(int action) {
		iAction = action;
	}

	/**
	 * @return Returns the iStatus.
	 */
	public synchronized int getStatus() {
		return iStatus;
	}

	/**
	 * @param status The iStatus to set.
	 */
	public synchronized void setStatus(int status) {
		iStatus = status;
	}

	/**
	 * @return Returns the iTransactionID.
	 */
	public synchronized long getTransactionID() {
		return iTransactionID;
	}

	/**
	 * @param transactionID The iTransactionID to set.
	 */
	public synchronized void setTransactionID(long transactionID) {
		iTransactionID = transactionID;
	}

	/**
	 * @return Returns the oBeginTimestamp.
	 */
	public synchronized Date getBeginTimestamp() {
		return oBeginTimestamp;
	}

	/**
	 * @param beginTimestamp The oBeginTimestamp to set.
	 */
	public synchronized void setBeginTimestamp(Date beginTimestamp) {
		oBeginTimestamp = beginTimestamp;
	}

	/**
	 * @return Returns the oEndTimestamp.
	 */
	public synchronized Date getEndTimestamp() {
		return oEndTimestamp;
	}

	/**
	 * @param endTimestamp The oEndTimestamp to set.
	 */
	public synchronized void setEndTimestamp(Date endTimestamp) {
		oEndTimestamp = endTimestamp;
	}

	/**
	 * @return Returns the oValue.
	 */
	public synchronized Serializable getValue() {
		return oValue;
	}

	/**
	 * @param value The oValue to set.
	 */
	public synchronized void setValue(Serializable value) {
		oValue = value;
	}

	/**
	 * @return Returns the strFilename.
	 */
	public String getFilename() {
		return strFilename;
	}

	/**
	 * @param strFilename The strFilename to set.
	 */
	public void setFilename(String strFilename) {
		this.strFilename = strFilename;
	}
	
}
