/**
 * 
 */
package marf.net.server.rmi.Classification;

import java.rmi.RemoteException;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.UnicastRemoteObject;

import marf.Configuration;
import marf.Classification.ClassificationException;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.net.server.delegates.Classification.BasicClassificationDelegate;
import marf.net.server.delegates.Classification.IClassificationDelegate;
import marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI;
import marf.net.server.rmi.util.RMIUtils;
import marf.util.Logger;
import marf.util.MARFException;


/**
 * RMI-specific classification servant.
 * 
 * @author mokhov
 *
 */
public class ClassificationServant
extends UnicastRemoteObject
implements IClassificationRMI
{
	/**
	 * Delegate for transactions, inter-server communication, and business logic. 
	 */
	private IClassificationDelegate oDelegate = null;

	/**
	 * For remote pipeline stage.
	 * Cannot be delegated. 
	 */
	private IFeatureExtractionRMI oFeatureExtractionRemote = null;
	
	/**
	 * @param port
	 * @param csf
	 * @param ssf
	 * @throws RemoteException
	 */
	public ClassificationServant(int port, RMIClientSocketFactory csf, RMIServerSocketFactory ssf)
	throws RemoteException
	{
		super(port, csf, ssf);
	}

	/**
	 * @param port
	 * @throws RemoteException
	 */
	public ClassificationServant(int port)
	throws RemoteException 
	{
		super(port);
	}

	/**
	 * @throws RemoteException
	 * @throws ClassificationException
	 */
	public ClassificationServant()
	throws RemoteException, ClassificationException, Exception 
	{
		super();
		new Logger("classification.rmi.log");
		this.oDelegate = new BasicClassificationDelegate();
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#classify()
	 */
	public boolean classify()
	throws RemoteException, ClassificationException
	{
//		return this.oDelegate.classify();
		return this.oDelegate.classify(this.oFeatureExtractionRemote.getFeaturesArray());
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#train()
	 */
	public boolean train()
	throws RemoteException, ClassificationException
	{
//		return this.oDelegate.train();
		return this.oDelegate.train(this.oFeatureExtractionRemote.getFeaturesArray());
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#classify(double[])
	 */
	public boolean classify(double[] padFeatureVector)
	throws RemoteException, ClassificationException
	{
		return this.oDelegate.classify(padFeatureVector);
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#init(marf.Configuration)
	 */
	public boolean init(Configuration poConfig)
	throws RemoteException, MARFException
	{
		return this.oDelegate.init(poConfig);
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#train(double[])
	 */
	public boolean train(double[] padFeatureVector)
	throws RemoteException, ClassificationException
	{
		return this.oDelegate.train(padFeatureVector);
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#getResult()
	 */
	public Result getResult()
	throws RemoteException
	{
		return this.oDelegate.getResult();
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#getResultSet()
	 */
	public ResultSet getResultSet()
	throws RemoteException
	{
		return this.oDelegate.getResultSet();
	}

	/**
	 * Local feature extraction.
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#getFeatureExtraction()
	 */
	public IFeatureExtraction getFeatureExtraction()
	throws RemoteException
	{
		return this.oDelegate.getFeatureExtraction();
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#setFeatureExtraction(marf.FeatureExtraction.IFeatureExtraction)
	 */
	public void setFeatureExtraction(IFeatureExtraction poFeatureExtraction)
	throws RemoteException
	{
		this.oDelegate.setFeatureExtraction(poFeatureExtraction);
	}

	/**
	 * Remote feature extraction.
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#getRemoteFeatureExtraction()
	 */
	public IFeatureExtractionRMI getRemoteFeatureExtraction()
	throws RemoteException
	{
		return this.oFeatureExtractionRemote;
	}

	/**
	 * @see marf.net.server.rmi.Classification.IClassificationRMI#setRemoteFeatureExtraction(marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI)
	 */
	public void setRemoteFeatureExtraction(IFeatureExtractionRMI poFeatureExtraction)
	throws RemoteException
	{
		this.oFeatureExtractionRemote = poFeatureExtraction;
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		RMIUtils.startRMIClassificationService();
	}
}

// EOF
