package marf.net.server.rmi.Classification;

import java.rmi.RemoteException;

import marf.Configuration;
import marf.Classification.ClassificationException;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.net.server.rmi.IRMIServer;
import marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI;
import marf.util.MARFException;


/**
 * <p>Remote Classification Interface.</p>
 *
 * $Id: IClassificationRMI.java,v 1.3 2006/08/16 00:52:26 mokhov Exp $
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.3 $
 * @since 0.3.0.6
 */
public interface IClassificationRMI
extends IRMIServer, Cloneable
{
	/**
	 * Interface source code revision.
	 */
	String MARF_INTERFACE_CODE_REVISION = "$Revision: 1.3 $";

	/* Classification API */

	/**
	 * @param poConfig
	 * @return
	 * @throws RemoteException
	 * @throws MARFException
	 */
	boolean init(Configuration poConfig)
	throws RemoteException, MARFException;

	/**
	 * Generic classification routine.
	 * @return <code>true</code> if classification was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was an error while classifying
	 */
	boolean classify()
	throws RemoteException, ClassificationException;

	/**
	 * Generic training routine for building/updating
	 * mean vectors in the training set.
	 *
	 * @return <code>true</code> if training was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was a problem while training
	 */
	boolean train()
	throws RemoteException, ClassificationException;

	/**
	 * Generic classification routine.
	 *
	 * @param padFeatureVector vector of features to compare with the stored ones
	 * @return <code>true</code> if classification was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was an error while classifying
	 * @since 0.3.0.6
	 */
	boolean classify(double[] padFeatureVector)
	throws RemoteException, ClassificationException;

	/**
	 * Generic training routine for building/updating
	 * mean vectors in the training set.
	 *
	 * @param padFeatureVector vector of features to train on
	 * @return <code>true</code> if training was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was a problem while training
	 * @since 0.3.0.6
	 */
	boolean train(double[] padFeatureVector)
	throws RemoteException, ClassificationException;

	/**
	 * Retrieves the likely classification result.
	 * If there were many, this will return the result with the
	 * highest statistical score or probability. The decision
	 * of whether to retrieve a maximum result (with maximum probability)
	 * or minimum result (with minimum distance) from the sample is
	 * left to be made by concrete implementations.
	 *
	 * @return Result object
	 */
	Result getResult()
	throws RemoteException;

	/**
	 * Retrieves the enclosed result set.
	 * @return the enclosed ResultSet object
	 */
	ResultSet getResultSet()
	throws RemoteException;

	/**
	 * Retrieves the features source.
	 * @return returns the FeatureExtraction reference
	 */
	IFeatureExtraction getFeatureExtraction()
	throws RemoteException;

	/**
	 * Allows setting the features source.
	 * @param poFeatureExtraction the FeatureExtraction object to set
	 */
	void setFeatureExtraction(IFeatureExtraction poFeatureExtraction)
	throws RemoteException;

	/**
	 * Retrieves the features source.
	 * @return returns the FeatureExtraction reference
	 */
	IFeatureExtractionRMI getRemoteFeatureExtraction()
	throws RemoteException;

	/**
	 * Allows setting the features source.
	 * @param poFeatureExtraction the FeatureExtraction object to set
	 */
	void setRemoteFeatureExtraction(IFeatureExtractionRMI poFeatureExtraction)
	throws RemoteException;
}

// EOF
