/**
 * 
 */
package marf.net.server.rmi.FeatureExtraction;

import java.rmi.RemoteException;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.UnicastRemoteObject;

import marf.FeatureExtraction.FeatureExtractionException;
import marf.Preprocessing.IPreprocessing;
import marf.net.server.delegates.FeatureExtraction.BasicFeatureExtractionDelegate;
import marf.net.server.delegates.FeatureExtraction.IFeatureExtractionDelegate;
import marf.net.server.rmi.Preprocessing.IPreprocessingRMI;
import marf.net.server.rmi.util.RMIUtils;
import marf.util.Logger;
import marf.util.MARFException;


/**
 * @author mokhov
 *
 */
public class FeatureExtractionServant
extends UnicastRemoteObject
implements IFeatureExtractionRMI
{
	private IFeatureExtractionDelegate oDelegate = null;
	
	private IPreprocessingRMI oPreprocessinRemote = null;

	/**
	 * @throws RemoteException
	 */
	public FeatureExtractionServant()
	throws RemoteException, Exception
	{
		super();
		new Logger("featureextraction.rmi.log");
		this.oDelegate = new BasicFeatureExtractionDelegate();
	}

	/**
	 * @param piPort
	 * @throws RemoteException
	 */
	public FeatureExtractionServant(int piPort)
	throws RemoteException
	{
		super(piPort);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param port
	 * @param csf
	 * @param ssf
	 * @throws RemoteException
	 */
	public FeatureExtractionServant(int port, RMIClientSocketFactory csf,
			RMIServerSocketFactory ssf) throws RemoteException {
		super(port, csf, ssf);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#init(int)
	 */
	public boolean init(int piFeatureExtractionMethod)
	throws RemoteException, MARFException
	{
		return this.oDelegate.init(piFeatureExtractionMethod);
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#extractFeatures()
	 */
	public boolean extractFeatures()
	throws RemoteException, FeatureExtractionException
	{
		return this.oDelegate.extractFeatures(this.oPreprocessinRemote.getSample().getSampleArray());
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#extractFeatures(double[])
	 */
	public boolean extractFeatures(double[] padSampleData)
	throws RemoteException, FeatureExtractionException
	{
		return this.oDelegate.extractFeatures(padSampleData);
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#getFeaturesArray()
	 */
	public double[] getFeaturesArray()
	throws RemoteException
	{
		return this.oDelegate.getFeaturesArray();
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#getPreprocessing()
	 */
	public IPreprocessing getPreprocessing()
	throws RemoteException
	{
		return this.oDelegate.getPreprocessing();
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#setPreprocessing(marf.Preprocessing.IPreprocessing)
	 */
	public void setPreprocessing(IPreprocessing poPreprocessing)
	throws RemoteException
	{
		this.oDelegate.setPreprocessing(poPreprocessing);
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#getRemotePreprocessing()
	 */
	public IPreprocessingRMI getRemotePreprocessing()
	throws RemoteException
	{
		return this.oPreprocessinRemote;
	}

	/**
	 * @see marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI#setRemotePreprocessing(marf.net.server.rmi.Preprocessing.IPreprocessingRMI)
	 */
	public void setRemotePreprocessing(IPreprocessingRMI poPreprocessing)
	throws RemoteException
	{
		this.oPreprocessinRemote = poPreprocessing;
	}

	
	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		RMIUtils.startRMIFeatureExtractionService();
	}

}

// EOF
