package marf.net.server.rmi.FeatureExtraction;

import java.rmi.RemoteException;

import marf.FeatureExtraction.FeatureExtractionException;
import marf.Preprocessing.IPreprocessing;
import marf.net.server.rmi.IRMIServer;
import marf.net.server.rmi.Preprocessing.IPreprocessingRMI;
import marf.util.MARFException;


/**
 * <p>Feature Extraction Interface.</p>
 *
 * $Id: IFeatureExtractionRMI.java,v 1.4 2006/09/17 20:06:29 mokhov Exp $
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.4 $
 * @since 0.3.0.6
 */
public interface IFeatureExtractionRMI
extends IRMIServer, Cloneable
{
	/**
	 * Interface source code revision.
	 */
	String MARF_INTERFACE_CODE_REVISION = "$Revision: 1.4 $";

	/**
	 * Initializes the module with the specified method.
	 * @param piFeatureExtractionMethod the business logic delegate code
	 * @return true if initialization was successful
	 * @throws RemoteException
	 * @throws MARFException
	 */
	boolean init(int piFeatureExtractionMethod)
	throws RemoteException, MARFException;

	/**
	 * Abstract feature extraction routine.
	 * @return boolean true if there were features extracted, false otherwise
	 * @throws FeatureExtractionException if there was an error while extracting features
	 * @throws RemoteException
	 */
	boolean extractFeatures()
	throws RemoteException, FeatureExtractionException;

	/**
	 * Abstract feature extraction routine.
	 * @param padSampleData the sample to extract features from
	 * @return boolean true if there were features extracted, false otherwise
	 * @throws FeatureExtractionException if there was an error while extracting features
	 * @throws RemoteException
	 */
	boolean extractFeatures(double[] padSampleData)
	throws RemoteException, FeatureExtractionException;

	/**
	 * Allows retrieval of the features.
	 * @return array of features (<code>double</code> values)
	 * @throws RemoteException
	 */
	double[] getFeaturesArray()
	throws RemoteException;

	/**
	 * Retrieves inner preprocessing reference.
	 * @return the preprocessing reference
	 * @throws RemoteException
	 */
	IPreprocessing getPreprocessing()
	throws RemoteException;

	/**
	 * Allows setting the source preprocessing module.
	 * @param poPreprocessing the preprocessing object to set
	 * @throws RemoteException
	 */
	void setPreprocessing(IPreprocessing poPreprocessing)
	throws RemoteException;

	/**
	 * Retrieves remote preprocessing reference.
	 * @return the preprocessing reference
	 * @throws RemoteException
	 */
	IPreprocessingRMI getRemotePreprocessing()
	throws RemoteException;

	/**
	 * Allows setting the remote source preprocessing module.
	 * @param poPreprocessing the preprocessing object to set
	 * @throws RemoteException
	 */
	void setRemotePreprocessing(IPreprocessingRMI poPreprocessing)
	throws RemoteException;
}

// EOF
