/**
 * 
 */
package marf.net.server.rmi.MARF;

import java.rmi.RemoteException;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.UnicastRemoteObject;

import marf.Configuration;
import marf.MARF;
import marf.Classification.ClassificationException;
import marf.Classification.IClassification;
import marf.FeatureExtraction.IFeatureExtraction;
import marf.Preprocessing.IPreprocessing;
import marf.Storage.ISampleLoader;
import marf.Storage.ModuleParams;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.Storage.Sample;
import marf.gui.WaveGrapher;
import marf.net.client.rmi.IRMIClient;
import marf.net.server.delegates.MARF.BasicMARFDelegate;
import marf.net.server.delegates.MARF.IMARFServerDelegate;
import marf.net.server.rmi.Classification.IClassificationRMI;
import marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI;
import marf.net.server.rmi.Preprocessing.IPreprocessingRMI;
import marf.net.server.rmi.Storage.ISampleLoaderRMI;
import marf.net.server.rmi.util.RMIUtils;
import marf.util.Debug;
import marf.util.MARFException;


/**
 * @author mokhov
 *
 */
public class MARFServant
extends UnicastRemoteObject
implements IMARFServerRMI, IRMIClient
{
	private IMARFServerDelegate oDelegate = null;
	
	private byte[] atRawAudioData = null;

	private ISampleLoaderRMI oSampleLoaderRemote = null;
	private IPreprocessingRMI oPreprocessingRemote = null;
	private IFeatureExtractionRMI oFeatureExtractionRemote = null;
	private IClassificationRMI oClassificationRemote = null;

	/**
	 * @throws RemoteException
	 */
	public MARFServant()
	throws RemoteException
	{
		super();
		this.oDelegate = new BasicMARFDelegate();
	}

	/**
	 * @param port
	 * @throws RemoteException
	 */
	public MARFServant(int port) throws RemoteException {
		super(port);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param port
	 * @param csf
	 * @param ssf
	 * @throws RemoteException
	 */
	public MARFServant(int port, RMIClientSocketFactory csf,
			RMIServerSocketFactory ssf) throws RemoteException {
		super(port, csf, ssf);
		// TODO Auto-generated constructor stub
	}

	
	
	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#checkSettings()
	 */
	public void checkSettings()
	throws RemoteException, MARFException
	{
		this.oDelegate.checkSettings();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getClassification()
	 */
	public IClassification getClassification()
	throws RemoteException
	{
		return this.oDelegate.getClassification();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getClassificationMethod()
	 */
	public int getClassificationMethod()
	throws RemoteException
	{
		return this.oDelegate.getClassificationMethod();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getClassificationPluginClass()
	 */
	public Class getClassificationPluginClass()
	throws RemoteException
	{
		return this.oDelegate.getClassificationPluginClass();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getConfig()
	 */
	public String getConfig()
	throws RemoteException
	{
		return this.oDelegate.getConfig();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getCurrentSubject()
	 */
	public int getCurrentSubject()
	throws RemoteException
	{
		return this.oDelegate.getCurrentSubject();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getDoubleVersion()
	 */
	public double getDoubleVersion()
	throws RemoteException
	{
		return this.oDelegate.getDoubleVersion();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getDumpSpectrogram()
	 */
	public boolean getDumpSpectrogram()
	throws RemoteException
	{
		return this.oDelegate.getDumpSpectrogram();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getDumpWaveGraph()
	 */
	public boolean getDumpWaveGraph()
	throws RemoteException
	{
		return this.oDelegate.getDumpWaveGraph();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getFeatureExtraction()
	 */
	public IFeatureExtraction getFeatureExtraction()
	throws RemoteException
	{
		return this.oDelegate.getFeatureExtraction();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getFeatureExtractionMethod()
	 */
	public int getFeatureExtractionMethod()
	throws RemoteException
	{
		return this.oDelegate.getFeatureExtractionMethod();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getFeatureExtractionPluginClass()
	 */
	public Class getFeatureExtractionPluginClass()
	throws RemoteException
	{
		return this.oDelegate.getFeatureExtractionPluginClass();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getIntVersion()
	 */
	public int getIntVersion()
	throws RemoteException
	{
		return this.oDelegate.getIntVersion();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getModuleParams()
	 */
	public ModuleParams getModuleParams()
	throws RemoteException
	{
		return this.oDelegate.getModuleParams();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getPreprocessing()
	 */
	public IPreprocessing getPreprocessing()
	throws RemoteException
	{
		return this.oDelegate.getPreprocessing();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getPreprocessingMethod()
	 */
	public int getPreprocessingMethod()
	throws RemoteException
	{
		return this.oDelegate.getPreprocessingMethod();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getPreprocessingPluginClass()
	 */
	public Class getPreprocessingPluginClass()
	throws RemoteException
	{
		return this.oDelegate.getPreprocessingPluginClass();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getResult()
	 */
	public Result getResult()
	throws RemoteException
	{
		return this.oDelegate.getResult();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getResultSet()
	 */
	public ResultSet getResultSet()
	throws RemoteException
	{
		return this.oDelegate.getResultSet();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getSample()
	 */
	public Sample getSample()
	throws RemoteException
	{
		return this.oDelegate.getSample();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getSampleFile()
	 */
	public String getSampleFile()
	throws RemoteException
	{
		return this.oDelegate.getSampleFile();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getSampleFormat()
	 */
	public int getSampleFormat()
	throws RemoteException
	{
		return this.oDelegate.getSampleFormat();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getSampleLoader()
	 */
	public ISampleLoader getSampleLoader()
	throws RemoteException
	{
		return this.oDelegate.getSampleLoader();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getSampleLoaderPluginClass()
	 */
	public Class getSampleLoaderPluginClass()
	throws RemoteException
	{
		return this.oDelegate.getSampleLoaderPluginClass();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getVersion()
	 */
	public String getVersion()
	throws RemoteException
	{
		return this.oDelegate.getVersion();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#queryResultID()
	 */
	public int queryResultID()
	throws RemoteException
	{
		return this.oDelegate.queryResultID();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#recognize()
	 */
	public void recognize()
	throws RemoteException, MARFException
	{
		// LOCAL
		//this.oDelegate.recognize();

		//THIS
		startRecognitionPipeline();

		Debug.debug("MARF: Classifying...");

		synchronized(oClassificationRemote)
		{
			if(oClassificationRemote.classify() == false)
			{
				throw new ClassificationException("Classification returned false.");
			}
		}
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setClassificationMethod(int)
	 */
	public void setClassificationMethod(int piClassificationMethod)
	throws RemoteException, MARFException
	{
		this.oDelegate.setClassificationMethod(piClassificationMethod);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setClassificationPluginClass(java.lang.Class)
	 */
	public void setClassificationPluginClass(Class poClass)
	throws RemoteException, MARFException
	{
		this.oDelegate.setClassificationPluginClass(poClass);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setClassificationPluginClass(java.lang.String)
	 */
	public void setClassificationPluginClass(String pstrClassName)
	throws RemoteException, MARFException
	{
		this.oDelegate.setClassificationPluginClass(pstrClassName);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setCurrentSubject(int)
	 */
	public void setCurrentSubject(int piSubjectID)
	throws RemoteException
	{
		this.oDelegate.setCurrentSubject(piSubjectID);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setDumpSpectrogram(boolean)
	 */
	public void setDumpSpectrogram(boolean pbDump)
	throws RemoteException
	{
		this.oDelegate.setDumpSpectrogram(pbDump);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setDumpWaveGraph(boolean)
	 */
	public void setDumpWaveGraph(boolean pbDump)
	throws RemoteException
	{
		this.oDelegate.setDumpWaveGraph(pbDump);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setFeatureExtractionMethod(int)
	 */
	public void setFeatureExtractionMethod(int piFeatureExtractionMethod)
	throws RemoteException, MARFException
	{
		this.oDelegate.setFeatureExtractionMethod(piFeatureExtractionMethod);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setFeatureExtractionPluginClass(java.lang.Class)
	 */
	public void setFeatureExtractionPluginClass(Class poClass)
	throws RemoteException, MARFException
	{
		this.oDelegate.setFeatureExtractionPluginClass(poClass);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setFeatureExtractionPluginClass(java.lang.String)
	 */
	public void setFeatureExtractionPluginClass(String pstrClassName)
	throws RemoteException, MARFException
	{
		this.oDelegate.setFeatureExtractionPluginClass(pstrClassName);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setModuleParams(marf.Storage.ModuleParams)
	 */
	public void setModuleParams(ModuleParams poModuleParams)
	throws RemoteException
	{
		this.oDelegate.setModuleParams(poModuleParams);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setPreprocessingMethod(int)
	 */
	public void setPreprocessingMethod(int piPreprocessingMethod)
	throws RemoteException, MARFException
	{
		this.oDelegate.setPreprocessingMethod(piPreprocessingMethod);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setPreprocessingPluginClass(java.lang.Class)
	 */
	public void setPreprocessingPluginClass(Class poClass)
	throws RemoteException, MARFException
	{
		this.oDelegate.setPreprocessingPluginClass(poClass);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setPreprocessingPluginClass(java.lang.String)
	 */
	public void setPreprocessingPluginClass(String pstrClassName)
	throws RemoteException, MARFException
	{
		this.oDelegate.setPreprocessingPluginClass(pstrClassName);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setSampleFile(java.lang.String)
	 */
	public void setSampleFile(String pstrFileName)
	throws RemoteException
	{
		this.oDelegate.setSampleFile(pstrFileName);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setSampleFile(java.lang.String, byte[])
	 */
	public void setSampleFile(String pstrFileName, byte[] patSampleData)
	throws RemoteException
	{
		this.oDelegate.setSampleFile(pstrFileName);
		this.atRawAudioData = patSampleData;
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setSampleFormat(int)
	 */
	public void setSampleFormat(int piSampleFormat)
	throws RemoteException
	{
		this.oDelegate.setSampleFormat(piSampleFormat);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setSampleLoaderPluginClass(java.lang.Class)
	 */
	public void setSampleLoaderPluginClass(Class poClass)
	throws RemoteException, MARFException
	{
		this.oDelegate.setSampleLoaderPluginClass(poClass);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setSampleLoaderPluginClass(java.lang.String)
	 */
	public void setSampleLoaderPluginClass(String pstrClassName)
	throws RemoteException, MARFException
	{
		this.oDelegate.setSampleLoaderPluginClass(pstrClassName);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setSamplesDir(java.lang.String)
	 */
	public void setSamplesDir(String pstrSamplesDir)
	throws RemoteException
	{
		this.oDelegate.setSamplesDir(pstrSamplesDir);
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#startRecognitionPipeline()
	 */
	public void startRecognitionPipeline()
	throws RemoteException, MARFException
	{
		try
		{
			// LOCAL
			//this.oDelegate.startRecognitionPipeline();
	
			/*
			 * Checking minimal required settings
			 */
			checkSettings();
	
			// XXX
			String sstrFileName = getSampleFile();
			boolean sbDumpWaveGraph = MARF.getDumpWaveGraph();
	
			/*
			 * Sample Loading Stage
			 */
			synchronized(sstrFileName)
			{
				Debug.debug("Loading sample \"" + getSampleFile() + "\"");
	
				//soSampleLoader = SampleLoaderFactory.create(siSampleFormat);
				oSampleLoaderRemote = RMIUtils.startRMISampleLoaderClient();
	
				synchronized(oSampleLoaderRemote)
				{
	//				soSample = soSampleLoader.loadSample(sstrFileName);
	//				Sample soSample = oSampleLoaderRemote.loadSample(sstrFileName);
	//				Sample soSample = oSampleLoaderRemote.loadSampleFromArray(this.atRawAudioData);
					Sample soSample = oSampleLoaderRemote.loadSample(this.atRawAudioData);
	
					/*
					 * Preprocessing Stage
					 */
					synchronized(soSample)
					{
						Debug.debug("Preprocessing...");
	//					soPreprocessing = PreprocessingFactory.create(siPreprocessingMethod, soSample);
	//					oPreprocessingRemote = IPreprocessingCORBAHelper.narrow(ORBUtils.startORBPreprocessingClient(astrOBRArgv));
	//					oPreprocessingRemote.init(MARF.getPreprocessingMethod(), new SampleHolder(soSample));
						oPreprocessingRemote = RMIUtils.startRMIPreprocessingClient();
						oPreprocessingRemote.init(MARF.getPreprocessingMethod(), soSample);
	
						synchronized(oPreprocessingRemote)
						{
							// TODO: [SM]: Should this be in the preprocessing itself somewhere?
	
							if(sbDumpWaveGraph)
							{
								Debug.debug("Dumping initial wave graph...");
	
								new WaveGrapher
								(
									soSample.getSampleArray(),
									0,
									soSample.getSampleArray().length,
									getSampleFile(),
									"initial"
								).dump();
							}
	
							Debug.debug("Invoking preprocess() of " + oPreprocessingRemote.getClass().getName());
							oPreprocessingRemote.preprocess();
							Debug.debug("Done preprocess() of " + oPreprocessingRemote.getClass().getName());
	
							if(sbDumpWaveGraph)
							{
								Debug.debug("Dumping preprocessed wave graph...");
	
								new WaveGrapher
								(
									soSample.getSampleArray(),
									0,
									soSample.getSampleArray().length,
									getSampleFile(),
									"preprocessed"
								).dump();
							}
	
							/*
							 * Feature Extraction Stage
							 */
							Debug.debug("Feature extraction...");
	
							oFeatureExtractionRemote = RMIUtils.startRMIFeatureExtractionClient();
							oFeatureExtractionRemote.setRemotePreprocessing(oPreprocessingRemote);
							oFeatureExtractionRemote.init(MARF.getFeatureExtractionMethod());
	
							synchronized(oFeatureExtractionRemote)
							{
								oFeatureExtractionRemote.extractFeatures();
	//							oFeatureExtractionRemote.extractFeaturesFromArray(oPreprocessingRemote.getSample().adSample);
	
								/*
								 * Classification Stage
								 */
								Debug.debug("Classification...");
	//							soClassification = ClassificationFactory.create(siClassificationMethod, soFeatureExtraction);
								oClassificationRemote = RMIUtils.startRMIClassificationClient();
								oClassificationRemote.setRemoteFeatureExtraction(oFeatureExtractionRemote);
								oClassificationRemote.init(MARF.getConfiguration());
	
								/*
								 * Classification ends in here, as it is continue in one
								 * way or the other in train() or recognize() depending on
								 * the curren run-time mode.
								 */
							}// feat
						}// prep
					}//sample itself
				}//sampleloader
			}//samplefilename
		}
		catch(MARFException e)
		{
			throw e;
		}
		catch(RemoteException e)
		{
			throw e;
		}
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new MARFException(e);
		}
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#streamedRecognition()
	 */
	public void streamedRecognition()
	throws RemoteException
	{
		this.oDelegate.streamedRecognition();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#train()
	 */
	public void train()
	throws RemoteException, MARFException
	{
		//LOCAL
		//this.oDelegate.train();

		//THIS
		if(getCurrentSubject() == MARF.UNSET)
		{
			throw new MARFException("Unset subject ID for training.");
		}

		startRecognitionPipeline();

		Debug.debug("MARF: Training...");

		synchronized(oClassificationRemote)
		{
			//XXX batch: soClassification.restore();

			if(oClassificationRemote.train() == false)
			{
				throw new ClassificationException("Training returned false.");
			}

			//XXX batch: soClassification.dump();
		}
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getClassificationRemote()
	 */
	public IClassificationRMI getClassificationRemote()
	throws RemoteException
	{
		return this.oClassificationRemote;
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getConfiguration()
	 */
	public Configuration getConfiguration()
	throws RemoteException
	{
		return this.oDelegate.getConfiguration();
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getRemoteFeatureExtraction()
	 */
	public IFeatureExtractionRMI getRemoteFeatureExtraction()
	throws RemoteException
	{
		return this.oFeatureExtractionRemote;
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getRemotePreprocessing()
	 */
	public IPreprocessingRMI getRemotePreprocessing()
	throws RemoteException
	{
		return this.oPreprocessingRemote;
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#getRemoteSampleLoader()
	 */
	public ISampleLoaderRMI getRemoteSampleLoader()
	throws RemoteException
	{
		return this.oSampleLoaderRemote;
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#setConfiguration(marf.Configuration)
	 */
	public Configuration setConfiguration(Configuration poConfiguration)
	throws RemoteException, MARFException
	{
		return this.oDelegate.setConfiguration(poConfiguration);
	}

	
	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		RMIUtils.startRMIMARFService();
	}
	
}

// EOF
