/**
 * 
 */
package marf.net.server.rmi.Preprocessing;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import marf.Preprocessing.PreprocessingException;
import marf.Storage.Sample;
import marf.net.server.delegates.Preprocessing.BasicPreprocessingDelegate;
import marf.net.server.delegates.Preprocessing.IPreprocessingDelegate;
import marf.net.server.rmi.util.RMIUtils;
import marf.util.Logger;
import marf.util.MARFException;


/**
 * @author mokhov
 *
 */
public class PreprocessingServant
extends UnicastRemoteObject
implements IPreprocessingRMI
{
	/**
	 * Business and transaction logic delegate.  
	 */
	private IPreprocessingDelegate oDelegate = null;
	
	/**
	 * @throws RemoteException
	 */
	public PreprocessingServant()
	throws RemoteException, Exception 
	{
		super();
		new Logger("preprocessing.rmi.log");
		this.oDelegate = new BasicPreprocessingDelegate();
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#init(int, marf.Storage.Sample)
	 */
	public boolean init(int piPreprocessingMethod, Sample poSample)
	throws RemoteException, MARFException
	{
//		this.oDelegate = PreprocessingFactory.create(piPreprocessingMethod, poSample);
//		return this.oDelegate != null;
		return this.oDelegate.init(piPreprocessingMethod, poSample);
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#preprocess()
	 */
	public boolean preprocess()
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.preprocess();
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#removeNoise()
	 */
	public boolean removeNoise()
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.removeNoise();
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#removeSilence()
	 */
	public boolean removeSilence()
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.removeSilence();
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#normalize()
	 */
	public boolean normalize()
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.normalize();
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#normalize(int)
	 */
	public boolean normalize(int piIndexFrom)
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.normalize(piIndexFrom);
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#normalize(int, int)
	 */
	public boolean normalize(int piIndexFrom, int piIndexTo)
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.normalize(piIndexFrom, piIndexFrom);
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#cropAudio(double, double)
	 */
	public boolean cropAudio(double pdStartingFrequency, double pdEndFrequency)
	throws RemoteException, PreprocessingException
	{
		return this.oDelegate.cropAudio(pdStartingFrequency, pdEndFrequency);
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#getSample()
	 */
	public Sample getSample()
	throws RemoteException
	{
		return this.oDelegate.getSample();
	}

	/**
	 * @see marf.net.server.rmi.Preprocessing.IPreprocessingRMI#setSample(marf.Storage.Sample)
	 */
	public void setSample(Sample poSample)
	throws RemoteException
	{
		this.oDelegate.setSample(poSample);
	}

	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone()
    throws CloneNotSupportedException
    {
		return this.oDelegate.clone();
    }

	
	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
		RMIUtils.startRMIPreprocessingService();
	}
}

// EOF

