/**
 * 
 */
package marf.net.server.rmi.Storage;

import java.io.File;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.rmi.server.RMIClientSocketFactory;
import java.rmi.server.RMIServerSocketFactory;
import java.rmi.server.UnicastRemoteObject;

import marf.Storage.Sample;
import marf.Storage.StorageException;
import marf.net.server.delegates.SampleLoader.BasicSampleLoaderDelegate;
import marf.net.server.delegates.SampleLoader.ISampleLoaderDelegate;
import marf.net.server.rmi.util.RMIUtils;
import marf.util.Logger;


/**
 * @author mokhov
 *
 */
public class SampleLoaderServant
extends UnicastRemoteObject
implements ISampleLoaderRMI
{
	/**
	 * Business and transaction logic delegate.
	 */
	private ISampleLoaderDelegate oDelegate = null;

	/**
	 * @throws RemoteException
	 */
	public SampleLoaderServant()
	throws RemoteException, Exception
	{
		super();

		new Logger("sampleloader.rmi.log");
		System.err.println("Sample Loader Logger Initialized");
		this.oDelegate = new BasicSampleLoaderDelegate();
		System.err.println("Sample Loader Logger Delegate Constructed");
	}

	/**
	 * @param port
	 * @throws RemoteException
	 */
	public SampleLoaderServant(int port) throws RemoteException {
		super(port);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @param port
	 * @param csf
	 * @param ssf
	 * @throws RemoteException
	 */
	public SampleLoaderServant(int port, RMIClientSocketFactory csf,
			RMIServerSocketFactory ssf) throws RemoteException {
		super(port, csf, ssf);
		// TODO Auto-generated constructor stub
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#readAudioData(double[])
	 */
	public int readAudioData(double[] padAudioData)
	throws RemoteException, StorageException
	{
		return this.oDelegate.readAudioData(padAudioData);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#writeAudioData(double[], int)
	 */
	public int writeAudioData(double[] padAudioData, int piWords)
	throws RemoteException, StorageException
	{
		return this.oDelegate.writeAudioData(padAudioData, piWords);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#loadSample(java.io.File)
	 */
	public Sample loadSample(File poInFile)
	throws RemoteException, StorageException
	{
		return this.oDelegate.loadSample(poInFile);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#loadSample(java.lang.String)
	 */
	public Sample loadSample(String pstrFilename)
	throws RemoteException, StorageException
	{
		return this.oDelegate.loadSample(pstrFilename);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#loadSample(byte[])
	 */
	public Sample loadSample(byte[] patFileAudioData)
	throws RemoteException, StorageException
	{
		return this.oDelegate.loadSample(patFileAudioData);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#loadSample(java.io.InputStream)
	 */
	public Sample loadSample(InputStream poAudioDataInputStream)
	throws RemoteException, StorageException
	{
		return this.oDelegate.loadSample(poAudioDataInputStream);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#saveSample(java.io.File)
	 */
	public void saveSample(File poOutFile)
	throws RemoteException, StorageException
	{
		this.oDelegate.saveSample(poOutFile);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#saveSample(java.lang.String)
	 */
	public void saveSample(String pstrFilename)
	throws RemoteException, StorageException
	{
		this.oDelegate.saveSample(pstrFilename);
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#updateSample()
	 */
	public void updateSample()
	throws RemoteException, StorageException
	{
		this.oDelegate.updateSample();
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#reset()
	 */
	public void reset()
	throws RemoteException, StorageException
	{
		this.oDelegate.reset();
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#getSampleSize()
	 */
	public long getSampleSize()
	throws RemoteException, StorageException
	{
		return this.oDelegate.getSampleSize();
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#getSample()
	 */
	public Sample getSample()
	throws RemoteException
	{
		return this.oDelegate.getSample();
	}

	/**
	 * @see marf.net.server.rmi.Storage.ISampleLoaderRMI#setSample(marf.Storage.Sample)
	 */
	public void setSample(Sample poSample)
	throws RemoteException
	{
		this.oDelegate.setSample(poSample);
	}

	/**
	 * @param argv
	 * @throws Exception
	 */
	public static void main(String[] argv)
	throws Exception
	{
//		try
		{
			RMIUtils.startRMISampleLoaderService();
		}
		/*catch(Throwable e)
		{
			System.err.println("Sample Loader Error Cause: " + e.getCause());
			e.printStackTrace(System.err);
		}*/
	}
}

// EOF
