/**
 * 
 */
package marf.net.server.rmi.util;

import java.io.FileInputStream;
import java.io.Serializable;
import java.rmi.Naming;
import java.rmi.RMISecurityManager;
import java.rmi.Remote;
import java.rmi.server.UnicastRemoteObject;
import java.util.Properties;


//import marf.net.server.rmi.Storage.SampleLoaderServant_Stub;
import marf.MARF;
import marf.Storage.Sample;
import marf.net.client.rmi.IRMIClient;
import marf.net.server.rmi.Classification.ClassificationServant;
import marf.net.server.rmi.Classification.ClassificationServant_Stub;
import marf.net.server.rmi.Classification.IClassificationRMI;
import marf.net.server.rmi.FeatureExtraction.FeatureExtractionServant;
//import marf.net.server.rmi.FeatureExtraction.FeatureExtractionServant_Stub;
import marf.net.server.rmi.FeatureExtraction.IFeatureExtractionRMI;
import marf.net.server.rmi.MARF.IMARFServerRMI;
import marf.net.server.rmi.MARF.MARFServant;
//import marf.net.server.rmi.MARF.MARFServant_Stub;
import marf.net.server.rmi.Preprocessing.IPreprocessingRMI;
import marf.net.server.rmi.Preprocessing.PreprocessingServant;
//import marf.net.server.rmi.Preprocessing.PreprocessingServant_Stub;
import marf.net.server.rmi.Storage.ISampleLoaderRMI;
import marf.net.server.rmi.Storage.SampleLoaderServant;
//import marf.net.server.rmi.Storage.SampleLoaderServant_Stub;
import marf.net.server.frontend.rmi.SpeakerIdent.ISpeakerIdentRMI;
import marf.net.server.frontend.rmi.SpeakerIdent.SpeakerIdentServant;
import marf.net.server.frontend.rmi.SpeakerIdent.SpeakerIdentServant_Stub;
//import marf.net.server.frontend.rmi.SpeakerIdent.SpeakerIdentServant_Stub;
import marf.net.server.rmi.IRMIServer;


/**
 * @author mokhov
 *
 */
public class RMIUtils
{

	private static final String SERVER_CLASSIFICATION_RMI = "IClassificationRMI";

	private static final String SERVER_FEATURE_EXTRACTION_RMI = "IFeatureExtractionRMI";

	private static final String SERVER_PREPROCESSING_RMI = "IPreprocessingRMI";

	private static final String SERVER_SAMPLE_LOADER_RMI = "ISampleLoaderRMI";

	private static final String SERVER_MARF_RMI = "IMARFServerRMI";

	private static final String SERVER_SPEAKER_IDENT_RMI = "ISpeakerIdentRMI";

	public static final int BASE_RMI_PORT = 20000;

	public static final int BASE_RMI_SPEAKER_IDENT_PORT = BASE_RMI_PORT + 10;
	public static final int BASE_RMI_LANG_IDENT_PORT = BASE_RMI_PORT + 20;
	public static final int BASE_RMI_MARF_PORT = BASE_RMI_PORT;
	public static final int BASE_RMI_SAMPLE_LOADER_PORT = BASE_RMI_PORT + MARF.WAV;
	public static final int BASE_RMI_PREPROCESSING_PORT = BASE_RMI_PORT + MARF.MIN_PREPROCESSING_METHOD;
	public static final int BASE_RMI_FEATURE_EXTRACTION_PORT = BASE_RMI_PORT + MARF.MIN_FEATUREEXTRACTION_METHOD;
	public static final int BASE_RMI_CLASSIFICATION_PORT = BASE_RMI_PORT + MARF.MIN_CLASSIFICATION_METHOD;

	public static final String SPEAKER_IDENT_FE_HOST_PROPERTY = "speakerident.rmi.fe.host"; 
	public static final String MARF_SERVER_HOST_PROPERTY = "marf.rmi.server.host"; 
	public static final String SAMPLE_LOADER_HOST_PROPERTY = "sampleloader.rmi.host"; 
	public static final String PREPROCESSING_HOST_PROPERTY = "preprocessing.rmi.host"; 
	public static final String FEATURE_EXTRACTION_HOST_PROPERTY = "featureextraction.rmi.host"; 
	public static final String CLASSIFICATION_HOST_PROPERTY = "classification.rmi.host"; 
	
	public static final String DEFAULT_HOST = "localhost";
	
	public static final String DEFAULT_HOST_PROPERTIES_FILE = "dmarf-hosts.properties";
	
	public static final Properties SERVICES_HOSTS = new Properties(); 
	
	static
	{
		try
		{
			SERVICES_HOSTS.load(new FileInputStream(DEFAULT_HOST_PROPERTIES_FILE));
			SERVICES_HOSTS.list(System.out);
		}
		catch(Exception e)
		{
			System.err.println("WARNING: " + e.getMessage() + ", using defaults...");

			SERVICES_HOSTS.setProperty(SPEAKER_IDENT_FE_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(MARF_SERVER_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(SAMPLE_LOADER_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(PREPROCESSING_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(FEATURE_EXTRACTION_HOST_PROPERTY, DEFAULT_HOST);
			SERVICES_HOSTS.setProperty(CLASSIFICATION_HOST_PROPERTY, DEFAULT_HOST);
		}
	}
	

	/**
	 * Intentionally empty and private. 
	 */
	private RMIUtils()
	{
	}
	
	public static String getSpeakerIdentFEHost()
	{
		return SERVICES_HOSTS.getProperty(SPEAKER_IDENT_FE_HOST_PROPERTY);
	}
	
	public static String getMARFServerHost()
	{
		return SERVICES_HOSTS.getProperty(MARF_SERVER_HOST_PROPERTY);
	}
	
	public static String getSampleLoaderHost()
	{
		return SERVICES_HOSTS.getProperty(SAMPLE_LOADER_HOST_PROPERTY);
	}
	
	public static String getPreprocessingHost()
	{
		return SERVICES_HOSTS.getProperty(PREPROCESSING_HOST_PROPERTY);
	}
	
	public static String getFeatureExtractionHost()
	{
		return SERVICES_HOSTS.getProperty(FEATURE_EXTRACTION_HOST_PROPERTY);
	}
	
	public static String getClassificationHost()
	{
		return SERVICES_HOSTS.getProperty(CLASSIFICATION_HOST_PROPERTY);
	}

	/**
	 * Intializes RMISecurityManager if no manager is set yet.
	 * @return the reference to the installed security manager.
	 */
	public static SecurityManager initializeSecurityManager()
	{
		// Attempt to set an RMI security manager
		if(System.getSecurityManager() == null)
 		{
 			System.setSecurityManager(new RMISecurityManager());
 			System.out.println("Set RMISecurityManager.");
        }
 		else
 		{
 			System.out.println("Already has a security manager: "+System.getSecurityManager().getClass().getName()+", so can't set the new one.");
 		}
		
		return System.getSecurityManager();
	}
	
	/**
	 * @return
	 * @throws Exception
	 */
	public static ISpeakerIdentRMI startRMISpeakerIdentClient()
	throws Exception
	{
		initializeSecurityManager();
		// Lookup the service
		return (ISpeakerIdentRMI)Naming.lookup("//" + getSpeakerIdentFEHost() + "/" + SERVER_SPEAKER_IDENT_RMI);
	}
	
	public static IMARFServerRMI startRMIMARFClient()
	throws Exception
	{
		initializeSecurityManager();
		return (IMARFServerRMI)Naming.lookup("//" + getMARFServerHost() + "/" + SERVER_MARF_RMI);
	}

	public static ISampleLoaderRMI startRMISampleLoaderClient()
	throws Exception
	{
		initializeSecurityManager();
		return (ISampleLoaderRMI)Naming.lookup("//" + getSampleLoaderHost() + "/" + SERVER_SAMPLE_LOADER_RMI);
	}
	
	public static IPreprocessingRMI startRMIPreprocessingClient()
	throws Exception
	{
		initializeSecurityManager();
		return (IPreprocessingRMI)Naming.lookup("//" + getPreprocessingHost() + "/" + SERVER_PREPROCESSING_RMI);
	}
	
	public static IFeatureExtractionRMI startRMIFeatureExtractionClient()
	throws Exception
	{
		initializeSecurityManager();
		return (IFeatureExtractionRMI)Naming.lookup("//" + getFeatureExtractionHost() + "/" + SERVER_FEATURE_EXTRACTION_RMI);
	}

	public static IClassificationRMI startRMIClassificationClient()
	throws Exception
	{
		initializeSecurityManager();
		return (IClassificationRMI)Naming.lookup("//" + getClassificationHost() + "/" + SERVER_CLASSIFICATION_RMI);
	}


	
	public static IRMIServer startRMISpeakerIdentService()
	throws Exception
	{
		return startRMIService(SERVER_SPEAKER_IDENT_RMI, SpeakerIdentServant.class);
	}

	
	public static IRMIServer startRMIMARFService()
	throws Exception
	{
		return startRMIService(SERVER_MARF_RMI, MARFServant.class);
	}
	
	/**
	 * @return
	 * @throws Exception
	 */
	public static IRMIServer startRMISampleLoaderService()
	throws Exception
	{
		return startRMIService(SERVER_SAMPLE_LOADER_RMI, SampleLoaderServant.class);
	}

	public static IRMIServer startRMIPreprocessingService()
	throws Exception
	{
		return startRMIService(SERVER_PREPROCESSING_RMI, PreprocessingServant.class);
	}

	public static IRMIServer startRMIFeatureExtractionService()
	throws Exception
	{
		return startRMIService(SERVER_FEATURE_EXTRACTION_RMI, FeatureExtractionServant.class);
	}

	public static IRMIServer startRMIClassificationService()
	throws Exception
	{
		return startRMIService(SERVER_CLASSIFICATION_RMI, ClassificationServant.class);
	}

	public static IRMIServer startRMIService(String pstrName, String pstrServantClass)
	throws Exception
	{
		return startRMIService(pstrName, Class.forName(pstrServantClass));
	}

	/**
	 * @param pstrName for name resolution in Naming
	 * @param poServantClass class of the servant to instantiate
	 * @return the created server/servant reference
	 * @throws Exception if class cannot be loaded or there is an error of registering the server in Naming
	 * @see Class#forName(java.lang.String)
	 * @see Naming#rebind(java.lang.String, java.rmi.Remote)
	 */
	public static IRMIServer startRMIService(String pstrName, Class poServantClass)
	throws Exception
	{
        System.out.println(pstrName + " (" + poServantClass.getName() + "): initializing security manager");
		initializeSecurityManager();

		// Instantiate the the servant
        IRMIServer oServer = (IRMIServer)poServantClass.newInstance();

        System.out.println("New instance created: " + oServer.getClass().getName());
        System.out.println("Types: ");
        System.out.println("\t" + pstrName + " is ISpeakerIdentRMI: " + (oServer instanceof ISpeakerIdentRMI));
        System.out.println("\t" + pstrName + " is IMARFServerRMI: " + (oServer instanceof IMARFServerRMI));
        System.out.println("\t" + pstrName + " is ISampleLoaderRMI: " + (oServer instanceof ISampleLoaderRMI));
        System.out.println("\t" + pstrName + " is IPreprocessingRMI: " + (oServer instanceof IPreprocessingRMI));
        System.out.println("\t" + pstrName + " is IFeatureExtractionRMI: " + (oServer instanceof IFeatureExtractionRMI));
        System.out.println("\t" + pstrName + " is IClassificationRMI: " + (oServer instanceof IClassificationRMI));

        System.out.println("\t" + IRMIClient.class.getName() + ": " + (oServer instanceof IRMIClient));
        System.out.println("\t" + Remote.class.getName() + ": " + (oServer instanceof Remote));
        System.out.println("\t" + Serializable.class.getName() + ": " + (oServer instanceof Serializable));
        System.out.println("\t" + UnicastRemoteObject.class.getName() + ": " + (oServer instanceof UnicastRemoteObject));
        System.out.println("\tNew Sample is: " + new Sample());
        System.out.println("\tStub 1 : " + SpeakerIdentServant_Stub.class.getName());
/*        System.out.println("\tStub 2 : " + SampleLoaderServant_Stub.class.getName() + ", " + new SampleLoaderServant_Stub(null));
        System.out.println("\tStub 3 : " + MARFServant_Stub.class.getName());
        System.out.println("\tStub 4 : " + PreprocessingServant_Stub.class.getName());
        System.out.println("\tStub 5 : " + FeatureExtractionServant_Stub .class.getName());
        System.out.println("\tStub 6 : " + ClassificationServant_Stub.class.getName());
*/
        // Register the service
        Naming.rebind(pstrName, oServer);
        System.out.println(pstrName + " (" + oServer.getClass().getName() + ") server ready");
		
		return oServer;
	}
}

// EOF
