package marf.net.server.ws.Classification;

import java.rmi.RemoteException;

import marf.Configuration;
import marf.Classification.ClassificationException;
import marf.Storage.Result;
import marf.Storage.ResultSet;
import marf.net.client.ws.IWSClient;
import marf.net.server.ws.IWSServer;
import marf.net.server.ws.util.RemoteObjectReference;
import marf.util.MARFException;


/**
 * <p>Remote WS Classification Endpoint Interface.</p>
 *
 * $Id: IClassificationWS.java,v 1.2 2006/09/17 20:06:35 mokhov Exp $
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.2 $
 * @since 0.3.0.6
 */
public interface IClassificationWS
extends IWSServer, IWSClient
{
	/* Classification API */

	/**
	 * @param poConfig
	 * @return
	 * @throws RemoteException
	 * @throws MARFException
	 */
	boolean init(Configuration poConfig)
	throws RemoteException, MARFException;

	/**
	 * Generic classification routine.
	 * @return <code>true</code> if classification was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was an error while classifying
	 */
	boolean classify()
	throws RemoteException, ClassificationException;

	/**
	 * Generic training routine for building/updating
	 * mean vectors in the training set.
	 *
	 * @return <code>true</code> if training was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was a problem while training
	 */
	boolean train()
	throws RemoteException, ClassificationException;

	/**
	 * Generic classification routine.
	 *
	 * @param padFeatureVector vector of features to compare with the stored ones
	 * @return <code>true</code> if classification was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was an error while classifying
	 * @since 0.3.0.6
	 */
	boolean classify(double[] padFeatureVector)
	throws RemoteException, ClassificationException;

	/**
	 * Generic training routine for building/updating
	 * mean vectors in the training set.
	 *
	 * @param padFeatureVector vector of features to train on
	 * @return <code>true</code> if training was successful; <code>false</code> otherwise
	 * @throws ClassificationException if there was a problem while training
	 * @since 0.3.0.6
	 */
	boolean train(double[] padFeatureVector)
	throws RemoteException, ClassificationException;

	/**
	 * Retrieves the likely classification result.
	 * If there were many, this will return the result with the
	 * highest statistical score or probability. The decision
	 * of whether to retrieve a maximum result (with maximum probability)
	 * or minimum result (with minimum distance) from the sample is
	 * left to be made by concrete implementations.
	 *
	 * @return Result object
	 */
	Result getResult()
	throws RemoteException;

	/**
	 * Retrieves the enclosed result set.
	 * @return the enclosed ResultSet object
	 */
	ResultSet getResultSet()
	throws RemoteException;

	/**
	 * Retrieves the features source.
	 * @return returns the FeatureExtraction reference
	 */
/*	IFeatureExtraction getFeatureExtraction()
	throws RemoteException;

	/**
	 * Allows setting the features source.
	 * @param poFeatureExtraction the FeatureExtraction object to set
	 */
/*	void setFeatureExtraction(IFeatureExtraction poFeatureExtraction)
	throws RemoteException;

	/**
	 * Retrieves the features source.
	 * @return returns the FeatureExtraction reference
	 */
	RemoteObjectReference getRemoteFeatureExtraction()
	throws RemoteException;

	/**
	 * Allows setting the features source.
	 * @param poFeatureExtraction the FeatureExtraction object to set
	 */
	void setRemoteFeatureExtraction(RemoteObjectReference poFeatureExtraction)
	throws RemoteException;
}

// EOF
