package marf.net.server.ws.FeatureExtraction;

import java.rmi.RemoteException;

import marf.FeatureExtraction.FeatureExtractionException;
import marf.net.client.ws.IWSClient;
import marf.net.server.ws.IWSServer;
import marf.net.server.ws.util.RemoteObjectReference;
import marf.util.MARFException;


/**
 * <p>Feature Extraction Interface.</p>
 *
 * $Id: IFeatureExtractionWS.java,v 1.1 2006/09/17 20:06:36 mokhov Exp $
 *
 * @author Serguei Mokhov
 * @version $Revision: 1.1 $
 * @since 0.3.0.6
 */
public interface IFeatureExtractionWS
extends IWSServer, IWSClient
{
	/**
	 * @param piFeatureExtractionMethod
	 * @return
	 * @throws RemoteException
	 * @throws MARFException
	 */
	boolean init(int piFeatureExtractionMethod)
	throws RemoteException, MARFException;

	/**
	 * Abstract feature extraction routine.
	 * @return boolean true if there were features extracted, false otherwise
	 * @throws FeatureExtractionException if there was an error while extracting features
	 */
	boolean extractFeatures()
	throws RemoteException, FeatureExtractionException;

	/**
	 * Abstract feature extraction routine.
	 * @param padSampleData the sample to extract features from
	 * @return boolean true if there were features extracted, false otherwise
	 * @throws FeatureExtractionException if there was an error while extracting features
	 */
	boolean extractFeatures(double[] padSampleData)
	throws RemoteException, FeatureExtractionException;

	/**
	 * Allows retrieval of the features.
	 * @return array of features (<code>double</code> values)
	 */
	double[] getFeaturesArray()
	throws RemoteException;

	/**
	 * Retrieves inner preprocessing reference.
	 * @return the preprocessing reference
	 */
	//IPreprocessing getPreprocessing()
	//throws RemoteException;

	/**
	 * Allows setting the source preprocessing module.
	 * @param poPreprocessing the preprocessing object to set
	 */
	//void setPreprocessing(IPreprocessing poPreprocessing)
	//throws RemoteException;

	/**
	 * Retrieves inner preprocessing reference.
	 * @return the preprocessing reference
	 */
	RemoteObjectReference getRemotePreprocessing()
	throws RemoteException;

	/**
	 * Allows setting the source preprocessing module.
	 * @param poPreprocessing the preprocessing object to set
	 */
	void setRemotePreprocessing(RemoteObjectReference poPreprocessing)
	throws RemoteException;
}

// EOF
