/**
 * 
 */
package marf.net.server.ws.MARF;

import java.rmi.RemoteException;

import marf.MARF;
import marf.Classification.ClassificationException;
import marf.Storage.Sample;
import marf.gui.WaveGrapher;
import marf.net.server.delegates.MARF.BasicMARFDelegate;
import marf.net.server.ws.Classification.IClassificationWS;
import marf.net.server.ws.FeatureExtraction.IFeatureExtractionWS;
import marf.net.server.ws.Preprocessing.IPreprocessingWS;
import marf.net.server.ws.Storage.ISampleLoaderWS;
import marf.net.server.ws.util.RemoteObjectReference;
import marf.net.server.ws.util.WSUtils;
import marf.util.Debug;
import marf.util.MARFException;

/**
 * @author mokhov
 *
 */
public class MARFServant extends BasicMARFDelegate implements IMARFServerWS {

	private byte[] atSampleData = null;

	private RemoteObjectReference oSampleLoaderRef = null;
	private RemoteObjectReference oPreprocessingRef = null;
	private RemoteObjectReference oFeatureExtractionRef = null;
	private RemoteObjectReference oClassificationRef = null;

	private ISampleLoaderWS oSampleLoaderRemote = null;
	private IPreprocessingWS oPreprocessingRemote = null;
	private IFeatureExtractionWS oFeatureExtractionRemote = null;
	private IClassificationWS oClassificationRemote = null;
	
	/**
	 * 
	 */
	public MARFServant() {
		super();
		// TODO Auto-generated constructor stub
	}

	/* (non-Javadoc)
	 * @see marf.net.server.ws.MARF.IMARFServerWS#setSampleFile(java.lang.String, byte[])
	 */
	public void setSampleFile(String pstrFileName, byte[] patSampleData)
	throws RemoteException
	{
		setSampleFile(pstrFileName);
		this.atSampleData = patSampleData;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.ws.MARF.IMARFServerWS#getRemoteSampleLoader()
	 */
	public RemoteObjectReference getRemoteSampleLoader() throws RemoteException {
		return this.oSampleLoaderRef;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.ws.MARF.IMARFServerWS#getRemotePreprocessing()
	 */
	public RemoteObjectReference getRemotePreprocessing()
			throws RemoteException {
		return this.oPreprocessingRef;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.ws.MARF.IMARFServerWS#getRemoteFeatureExtraction()
	 */
	public RemoteObjectReference getRemoteFeatureExtraction()
			throws RemoteException {
		return this.oFeatureExtractionRef;
	}

	/* (non-Javadoc)
	 * @see marf.net.server.ws.MARF.IMARFServerWS#getClassificationRemote()
	 */
	public RemoteObjectReference getClassificationRemote()
			throws RemoteException {
		return this.oClassificationRef;
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#recognize()
	 */
	public void recognize()
	throws MARFException
	{
		try
		{
			// LOCAL
			//this.oDelegate.recognize();
	
			//THIS
			startRecognitionPipeline();
	
			Debug.debug("MARF: Classifying...");
	
			synchronized(oClassificationRemote)
			{
				if(oClassificationRemote.classify() == false)
				{
					throw new ClassificationException("Classification returned false.");
				}
			}
		}
		catch(RemoteException e)
		{
			e.printStackTrace(System.err);
			throw new MARFException(e);
		}
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#train()
	 */
	public void train()
	throws MARFException
	{
		try
		{
			//LOCAL
			//this.oDelegate.train();
	
			//THIS
			if(getCurrentSubject() == MARF.UNSET)
			{
				throw new MARFException("Unset subject ID for training.");
			}
	
			startRecognitionPipeline();
	
			Debug.debug("MARF: Training...");
	
			synchronized(oClassificationRemote)
			{
				//XXX batch: soClassification.restore();
	
				if(oClassificationRemote.train() == false)
				{
					throw new ClassificationException("Training returned false.");
				}
	
				//XXX batch: soClassification.dump();
			}
		}
		catch(RemoteException e)
		{
			e.printStackTrace(System.err);
			throw new MARFException(e);
		}
	}

	/**
	 * @see marf.net.server.rmi.MARF.IMARFServerRMI#startRecognitionPipeline()
	 */
	public void startRecognitionPipeline()
	throws MARFException
	{
		try
		{
			// LOCAL
			//this.oDelegate.startRecognitionPipeline();
	
			/*
			 * Checking minimal required settings
			 */
			checkSettings();
	
			// XXX
			String sstrFileName = getSampleFile();
			boolean sbDumpWaveGraph = MARF.getDumpWaveGraph();
	
			/*
			 * Sample Loading Stage
			 */
			synchronized(sstrFileName)
			{
				Debug.debug("Loading sample \"" + getSampleFile() + "\"");
	
				//soSampleLoader = SampleLoaderFactory.create(siSampleFormat);
				oSampleLoaderRemote = WSUtils.connectToWSSampleLoaderService();
	
				synchronized(oSampleLoaderRemote)
				{
	//				soSample = soSampleLoader.loadSample(sstrFileName);
	//				Sample soSample = oSampleLoaderRemote.loadSample(sstrFileName);
	//				Sample soSample = oSampleLoaderRemote.loadSampleFromArray(this.atRawAudioData);
					Sample soSample = oSampleLoaderRemote.loadSample(this.atSampleData);
	
					/*
					 * Preprocessing Stage
					 */
					synchronized(soSample)
					{
						Debug.debug("Preprocessing...");
	//					soPreprocessing = PreprocessingFactory.create(siPreprocessingMethod, soSample);
	//					oPreprocessingRemote = IPreprocessingCORBAHelper.narrow(ORBUtils.startORBPreprocessingClient(astrOBRArgv));
	//					oPreprocessingRemote.init(MARF.getPreprocessingMethod(), new SampleHolder(soSample));
						oPreprocessingRemote = WSUtils.connectToWSPreprocessingService();
						oPreprocessingRemote.init(MARF.getPreprocessingMethod(), soSample);
	
						synchronized(oPreprocessingRemote)
						{
							// TODO: [SM]: Should this be in the preprocessing itself somewhere?
	
							if(sbDumpWaveGraph)
							{
								Debug.debug("Dumping initial wave graph...");
	
								new WaveGrapher
								(
									soSample.getSampleArray(),
									0,
									soSample.getSampleArray().length,
									getSampleFile(),
									"initial"
								).dump();
							}
	
							Debug.debug("Invoking preprocess() of " + oPreprocessingRemote.getClass().getName());
							oPreprocessingRemote.preprocess();
							Debug.debug("Done preprocess() of " + oPreprocessingRemote.getClass().getName());
	
							if(sbDumpWaveGraph)
							{
								Debug.debug("Dumping preprocessed wave graph...");
	
								new WaveGrapher
								(
									soSample.getSampleArray(),
									0,
									soSample.getSampleArray().length,
									getSampleFile(),
									"preprocessed"
								).dump();
							}
	
							/*
							 * Feature Extraction Stage
							 */
							Debug.debug("Feature extraction...");
	
							oFeatureExtractionRemote = WSUtils.connectToWSFeatureExtractionService();
							oPreprocessingRef = new RemoteObjectReference(MARF.getPreprocessingMethod(), WSUtils.getPreprocessingHost());
							oFeatureExtractionRemote.setRemotePreprocessing(oPreprocessingRef);
							oFeatureExtractionRemote.init(MARF.getFeatureExtractionMethod());
	
							synchronized(oFeatureExtractionRemote)
							{
								oFeatureExtractionRemote.extractFeatures();
	//							oFeatureExtractionRemote.extractFeaturesFromArray(oPreprocessingRemote.getSample().adSample);
	
								/*
								 * Classification Stage
								 */
								Debug.debug("Classification...");
	//							soClassification = ClassificationFactory.create(siClassificationMethod, soFeatureExtraction);
								oClassificationRemote = WSUtils.connectToWSClassificationService();
								oFeatureExtractionRef = new RemoteObjectReference(MARF.getFeatureExtractionMethod(), WSUtils.getFeatureExtractionHost());
								oClassificationRemote.setRemoteFeatureExtraction(oFeatureExtractionRef);
								oClassificationRemote.init(MARF.getConfiguration());
	
								/*
								 * Classification ends in here, as it is continue in one
								 * way or the other in train() or recognize() depending on
								 * the curren run-time mode.
								 */
							}// feat
						}// prep
					}//sample itself
				}//sampleloader
			}//samplefilename
		}
		catch(MARFException e)
		{
			throw e;
		}
/*		catch(RemoteException e)
		{
			throw e;
		}*/
		catch(Exception e)
		{
			e.printStackTrace(System.err);
			throw new MARFException(e);
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
