/**
 * 
 */
package marf.net.server.ws.util;

import java.io.FileInputStream;
import java.util.Properties;

import marf.MARF;
import marf.net.server.frontend.ws.SpeakerIdent.ISpeakerIdentWS;
import marf.net.server.frontend.ws.SpeakerIdent.ISpeakerIdentWS_Service_Impl;
import marf.net.server.ws.Classification.IClassificationWS;
import marf.net.server.ws.Classification.IClassificationWS_Service_Impl;
import marf.net.server.ws.FeatureExtraction.IFeatureExtractionWS;
import marf.net.server.ws.FeatureExtraction.IFeatureExtractionWS_Service_Impl;
import marf.net.server.ws.MARF.IMARFServerWS;
import marf.net.server.ws.MARF.IMARFWS_Service_Impl;
import marf.net.server.ws.Preprocessing.IPreprocessingWS;
import marf.net.server.ws.Preprocessing.IPreprocessingWS_Service_Impl;
import marf.net.server.ws.Storage.ISampleLoaderWS;
import marf.net.server.ws.Storage.ISampleLoaderWS_Service_Impl;


/**
 * @author mokhov
 *
 */
public class WSUtils
{

	private static final String SERVER_CLASSIFICATION_WS = "IClassificationWS";

	private static final String SERVER_FEATURE_EXTRACTION_WS = "IFeatureExtractionWS";

	private static final String SERVER_PREPROCESSING_WS = "IPreprocessingWS";

	private static final String SERVER_SAMPLE_LOADER_WS = "ISampleLoaderWS";

	private static final String SERVER_MARF_WS = "IMARFServerWS";

	private static final String SERVER_SPEAKER_IDENT_WS = "ISpeakerIdentWS";

	public static final int BASE_WS_PORT = 30000;

	public static final int BASE_WS_SPEAKER_IDENT_PORT = BASE_WS_PORT + 10;
	public static final int BASE_WS_LANG_IDENT_PORT = BASE_WS_PORT + 20;
	public static final int BASE_WS_MARF_PORT = BASE_WS_PORT;
	public static final int BASE_WS_SAMPLE_LOADER_PORT = BASE_WS_PORT + MARF.WAV;
	public static final int BASE_WS_PREPROCESSING_PORT = BASE_WS_PORT + MARF.MIN_PREPROCESSING_METHOD;
	public static final int BASE_WS_FEATURE_EXTRACTION_PORT = BASE_WS_PORT + MARF.MIN_FEATUREEXTRACTION_METHOD;
	public static final int BASE_WS_CLASSIFICATION_PORT = BASE_WS_PORT + MARF.MIN_CLASSIFICATION_METHOD;

	public static final String SPEAKER_IDENT_FE_HOST_PROPERTY = "speakerident.ws.fe.host"; 
	public static final String MARF_SERVER_HOST_PROPERTY = "marf.ws.server.host"; 
	public static final String SAMPLE_LOADER_HOST_PROPERTY = "sampleloader.ws.host"; 
	public static final String PREPROCESSING_HOST_PROPERTY = "preprocessing.ws.host"; 
	public static final String FEATURE_EXTRACTION_HOST_PROPERTY = "featureextraction.ws.host"; 
	public static final String CLASSIFICATION_HOST_PROPERTY = "classification.ws.host"; 
	
	public static final String DEFAULT_HOST = "http://localhost:8080/dmarf/";
	
	public static final String DEFAULT_HOST_PROPERTIES_FILE = "dmarf-hosts.properties";
	
	public static final Properties SERVICES_HOSTS = new Properties(); 
	
	static
	{
		try
		{
			SERVICES_HOSTS.load(new FileInputStream(DEFAULT_HOST_PROPERTIES_FILE));
			SERVICES_HOSTS.list(System.out);
		}
		catch(Exception e)
		{
			System.err.println("WARNING: " + e.getMessage() + ", using defaults...");

			SERVICES_HOSTS.setProperty(SPEAKER_IDENT_FE_HOST_PROPERTY, DEFAULT_HOST + "speakerident");
			SERVICES_HOSTS.setProperty(MARF_SERVER_HOST_PROPERTY, DEFAULT_HOST + "marf");
			SERVICES_HOSTS.setProperty(SAMPLE_LOADER_HOST_PROPERTY, DEFAULT_HOST + "sampleloader");
			SERVICES_HOSTS.setProperty(PREPROCESSING_HOST_PROPERTY, DEFAULT_HOST + "preprocessing");
			SERVICES_HOSTS.setProperty(FEATURE_EXTRACTION_HOST_PROPERTY, DEFAULT_HOST + "featureextraction");
			SERVICES_HOSTS.setProperty(CLASSIFICATION_HOST_PROPERTY, DEFAULT_HOST + "classification");
		}
	}
	

	/**
	 * Intentionally empty and private. 
	 */
	private WSUtils()
	{
	}
	
	public static String getSpeakerIdentFEHost()
	{
		return SERVICES_HOSTS.getProperty(SPEAKER_IDENT_FE_HOST_PROPERTY);
	}
	
	public static String getMARFServerHost()
	{
		return SERVICES_HOSTS.getProperty(MARF_SERVER_HOST_PROPERTY);
	}
	
	public static String getSampleLoaderHost()
	{
		return SERVICES_HOSTS.getProperty(SAMPLE_LOADER_HOST_PROPERTY);
	}
	
	public static String getPreprocessingHost()
	{
		return SERVICES_HOSTS.getProperty(PREPROCESSING_HOST_PROPERTY);
	}
	
	public static String getFeatureExtractionHost()
	{
		return SERVICES_HOSTS.getProperty(FEATURE_EXTRACTION_HOST_PROPERTY);
	}
	
	public static String getClassificationHost()
	{
		return SERVICES_HOSTS.getProperty(CLASSIFICATION_HOST_PROPERTY);
	}

	
	/**
	 * @return
	 * @throws Exception
	 */
	public static ISpeakerIdentWS connectToWSSpeakerIdentClient()
	throws Exception
	{
		ISpeakerIdentWS_Service_Impl oService = new ISpeakerIdentWS_Service_Impl();
		RemoteObjectReference oRemoteService = new RemoteObjectReference();
		oRemoteService.strURL = getSpeakerIdentFEHost();
		return (ISpeakerIdentWS)oRemoteService.connect(oService);
	}
	
	public static IMARFServerWS connectToWSMARFService()
	throws Exception
	{
		IMARFWS_Service_Impl oService = new IMARFWS_Service_Impl();
		RemoteObjectReference oRemoteService = new RemoteObjectReference();
		oRemoteService.strURL = getMARFServerHost();
		return (IMARFServerWS)oRemoteService.connect(oService);
	}

	public static ISampleLoaderWS connectToWSSampleLoaderService()
	throws Exception
	{
		ISampleLoaderWS_Service_Impl oService = new ISampleLoaderWS_Service_Impl();
		RemoteObjectReference oRemoteService = new RemoteObjectReference();
		oRemoteService.strURL = getSampleLoaderHost();
		return (ISampleLoaderWS)oRemoteService.connect(oService);
	}
	
	/**
	 * @return
	 * @throws Exception
	 */
	public static IPreprocessingWS connectToWSPreprocessingService()
	throws Exception
	{
		RemoteObjectReference oRemoteService = new RemoteObjectReference();
		oRemoteService.strURL = getPreprocessingHost();
		return connectToWSPreprocessingService(oRemoteService);
	}
	
	public static IPreprocessingWS connectToWSPreprocessingService(RemoteObjectReference poRemoteService)
	throws Exception
	{
		IPreprocessingWS_Service_Impl oService = new IPreprocessingWS_Service_Impl();
		return (IPreprocessingWS)poRemoteService.connect(oService);
	}

	public static IFeatureExtractionWS connectToWSFeatureExtractionService()
	throws Exception
	{
		RemoteObjectReference oRemoteService = new RemoteObjectReference();
		oRemoteService.strURL = getFeatureExtractionHost();
		return connectToWSFeatureExtractionService(oRemoteService);
	}

	public static IFeatureExtractionWS connectToWSFeatureExtractionService(RemoteObjectReference poRemoteService)
	throws Exception
	{
		IFeatureExtractionWS_Service_Impl oService = new IFeatureExtractionWS_Service_Impl();
		return (IFeatureExtractionWS)poRemoteService.connect(oService);
	}

	public static IClassificationWS connectToWSClassificationService()
	throws Exception
	{
		IClassificationWS_Service_Impl oService = new IClassificationWS_Service_Impl();
		RemoteObjectReference oRemoteService = new RemoteObjectReference();
		oRemoteService.strURL = getFeatureExtractionHost();
		return (IClassificationWS)oRemoteService.connect(oService);
	}

}

// EOF
